from dungeonsheets.features.features import Feature, FeatureSelector
from dungeonsheets.features.ranger import Dueling, TwoWeaponFighting


# PHB
class BardicInspiration(Feature):
    """You can inspire others through stirring words or music. To do so, you use a
    bonus action on your turn to choose one creature other than yourself within
    60 feet of you who can hear you. That creature gains one Bardic Inspiration
    die, a d6.

    Once within the next 10 minutes, the creature can roll the die and add the
    number rolled to one ability check, attack roll, or saving throw it
    makes. The creature can wait until after it rolls the d20 before deciding
    to use the Bardic Inspiration die, but must decide before the DM says
    whether the roll succeeds or fails. Once the Bardic Inspiration die is
    rolled, it is lost. A creature can have only one Bardic Inspiration die at
    a time

    You can use this feature a number of times equal to your Charisma modifier
    (a minimum of once). You regain any expended uses when you finish a long
    rest.

    Your Bardic Inspiration die changes when you reach certain levels in this
    class. The die becomes a d8 at 5th level, a d10 at 10th level, and a d12
    at 15th level.

    """

    _name = "Bardic Inspiration"
    source = "Bard"

    @property
    def name(self):
        level = self.owner.Bard.level
        die = "d6"
        if level >= 5:
            die = "d8"
        if level >= 10:
            die = "d10"
        if level >= 15:
            die = "d12"
        num = max(1, self.owner.charisma.modifier)
        rest = "LR"
        if level >= 5:  # font of inspiration
            rest = "SR"
        return self._name + " ({:d}{:s}/{:s})".format(num, die, rest)


class JackOfAllTrades(Feature):
    """Starting at 2nd level, you can add half your proficiency bonus, rounded
    down, to any ability check you make that doesn't already include your
    proficiency bonus. (Included in stats on Character Sheet above).

    """

    name = "Jack of All Trades"
    source = "Bard"


class SongOfRest(Feature):
    """Beginning at 2nd level, you can use soothing music or oration to help
    revitalize your wounded allies during a short rest. If you or any friendly
    creatures who can hear your performance regain hit points at the end of the
    short rest, each of those creatures regains an extra 1d6 hit points. The
    extra hit points increase when you reach certain levels in this class: to
    1d8 at 9th level, to 1d10 at 13th level, and to 1d12 at 17th level.

    """

    _name = "Song of Rest"
    source = "Bard"

    @property
    def name(self):
        level = self.owner.Bard.level
        die = " (1d6)"
        if level >= 9:
            die = " (1d8)"
        if level >= 13:
            die = " (1d10)"
        if level >= 17:
            die = " (1d12)"
        return self._name + die


class BardExpertise(Feature):
    """At 3rd level, choose two of your skill proficiencies. Your proficiency
    bonus is doubled for any ability check you make that uses either of the
    chosen proficiencies. At 10th level, you can choose another two skill
    proficiencies to gain this benefit.

    Add these skills to "skill_expertise" in your character.py file

    """

    name = "Expertise"
    source = "Bard"


class FontOfInspiration(Feature):
    """Beginning when you reach 5th level, you regain all of your expended uses of
    Bardic Inspiration when you finish a short or long rest.

    """

    name = "Font of Inspiration"
    source = "Bard"


class Countercharm(Feature):
    """At 6th level, you gain the ability to use musical notes or words of power
    to disrupt mind-influencing effects. As an action, you can start a
    performance that lasts until the end of your next turn. During that time,
    you and any friendly creatures within 30 feet of you have advantage on
    saving throws against being frightened or charmed. A creature must be able
    to hear you to gain this benefit. The performance ends early if you are
    incapacitated or silenced or if you voluntarily end it (no action
    required).

    """

    name = "Countercharm"
    source = "Bard"


class MagicalSecrets(Feature):
    """By 10th level, you have plundered magical knowledge from a wide spectrum of
    disciplines. Choose two spells from any class, including Bard. A spell
    you choose must be of a level you can cast, as shown on the Bard table, or
    a cantrip. The chosen spells count as bard spells for you and are included
    in the number in the Spells Known column of the Bard table. You learn two
    additional spells from any class at 14th level and again at 18th level.

    """

    name = "Magical Secrets"
    source = "Bard"


class SuperiorInspiration(Feature):
    """At 20th level, when you roll initiative and have no uses of Bardic
    Inspiration left, you regain one use.

    """

    name = "Superior Inspiration"
    source = "Bard"


# College of Lore
class LoreProficiencies(Feature):
    """When you join the College of Lore at 3rd level, you gain proficiency with
    three skills o f your choice.

    """

    name = "Bonus Proficiencies"
    source = "Bard (College of Lore)"


class CuttingWords(Feature):
    """Also at 3rd level, you learn how to use your wit to distract, confuse, and
    otherwise sap the confidence and competence of others. When a creature that
    you can see within 60 feet of you makes an attack roll, an ability check,
    or a damage roll, you can use your reaction to expend one of your uses of
    Bardic Inspiration, rolling a Bardic Inspiration die and subtracting the
    number rolled from the creature's roll. You can choose to use this feature
    after the creature makes its roll, but before the DM determines whether the
    attack roll or ability check succeeds or fails, or before the creature
    deals its damage. The creature is immune if it can't hear you or if it's
    immune to being charmed.

    """

    name = "Cutting Words"
    source = "Bard (College of Lore)"


class AdditionalMagicalSecrets(Feature):
    """At 6th level, you learn two spells of your choice from any class. A spell
    you choose must be of a level you can cast, as shown on the Bard table, or
    a cantrip. The chosen spells count as bard spells for you but don't count
    against the number of bard spells you know.

    """

    name = "Additional Magical Secrets"
    source = "Bard (College of Lore)"


class PeerlessSkill(Feature):
    """Starting at 14th level, when you make an ability check, you can expend one
    use of Bardic Inspiration. Roll a Bardic Inspiration die and add the number
    rolled to your ability check. You can choose to do so after you roil the
    die for the ability check, but before the DM tells you whether you succeed
    or fail

    """

    name = "Peerless Skill"
    source = "Bard (College of Lore)"


# College of Valor
class CombatInspiration(Feature):
    """Also at 3rd level, you learn to inspire others in battle. A creature that
    has a Bardic Inspiration die from you can roll that die and add the number
    rolled to a weapon damage roll it just made. Alternatively, when an attack
    roll is made against the creature, it can use its reaction to roll the
    Bardic Inspiration die and add the number rolled to its AC against that
    attack, after seeing the roll but before knowing whether it hits or misses

    """

    name = "Combat Inspiration"
    source = "Bard (College of Valor)"


class BardExtraAttack(Feature):
    """Starting at 6th level, you can attack twice, instead of once, whenever you
    take the Attack action on your turn

    """

    name = "Extra Attack (x2)"
    source = "Bard (College of Valor)"


class BardBattleMagic(Feature):
    """At 14th level, you have mastered the art of weaving spellcasting and weapon
    use into a single harmonious act. When you use your action to cast a bard
    spell, you can make one weapon attack as a bonus action

    """

    name = "Battle Magic"
    source = "Bard (College of Valor)"


# XgtE - College of Glamour:
class MantleOfInspiration(Feature):
    """When you join the College of Glamour at 3rd level, you gain the
    ability to weave a song of fey magic that imbues your allies with
    vigor and speed. As a bonus action, you can expend one use of your
    Bardic Inspiration to grant yourself a wondrous appearance. When
    you do so, choose a number of creatures you can see and that can
    see you within 60 feet of you, up to a number equal to your
    Charisma modifier (minimum of one). Each of them gains 5 temporary
    hit points. When a creature gains these temporary hit points, it
    can immediately use its reaction to move up to its speed, without
    provoking opportunity attacks. The number of temporary hit points
    increases when you reach certain levels in this class, increasing
    to 8 at 5th level, 11 at 10th level, and 14 at 15th level.

    """

    _name = "Mantle of Inspiration"
    source = "Bard (College of Glamour)"

    @property
    def name(self):
        level = self.owner.Bard.level
        hp = 5 + 3 * (level // 5)
        return self._name + " ({:d}HP)".format(hp)


class EnthrallingPerformance(Feature):
    """Starting at 3rd level, you can charge your performance with seductive, fey
    magic. If you perform for at least 1 minute, you can attempt to inspire
    wonder in your audience by singing, reciting a poem, or dancing. At the end
    of the performance, choose a number of humanoids within 60 feet ofyou who
    watched and listened to all of it, up to a number equal tO your Charisma
    modifier (minimum of one). Each target must succeed on a Wisdom saving
    throw against your spell save DC or be charmed by you. While charmed in
    this way, the target idolizes you, it speaks glowingly Of you to anyone who
    talks to it, and it hinders anyone who Opposes you, although it avoids
    violence unless it was already inclined to fight on your behalf. This
    effect ends on a target after 1 hour, if it takes any damage, if you attack
    it, or if it witnesses you attacking or damaging any of its allies.

    If a target succeeds on its saving throw, the target has no hint that you
    tried to charm it. Once you use this feature, you can't use it again until
    you finish a short or long rest

    """

    name = "Enthralling Performance"
    source = "Bard (College of Glamour)"


class MantleOfMajesty(Feature):
    """At 6th level, you gain the ability to cloak yourself in a fey magic that
    makes Others want to serve you. As a bonus action, you cast command,
    without expending a spell slot, and you take on an appearance of unearthly
    beauty for 1 minute or until your concentration ends (as if you were
    concentrating on a spell). During this time, you can cast command as a
    bonus action on each of your turns, without expending a spell slot.

    Any creature charmed by you automatically failfs its saving throw against
    the command you cast with this feature. Once you use this feature, you
    can't use it again until you finish a long rest

    """

    name = "Mantle of Majesty"
    source = "Bard (College of Glamour)"


class UnbreakableMajesty(Feature):
    """At 14th level, your appearance permanently gains an otherworldly aspect that
    makes you look more lovely and fierce. In addition, as a bonus action, you
    can assume a magically majestic presence for 1 minute or until you are
    incapacitated. For the duration, whenever any creature tries to attack you
    for the first time on a turn, the at- tacker must make a Charisma saving
    throw against your spell save DC. On a failed save, it can't attack you on
    this turn, and it must choose a new target for its attack or the attack is
    wasted.

    On a successful save, it can attack you on this turn, but it has
    disadvantage on any saving throw it makes against your spells on your next
    turn. Once you assume this majestic presence, you can't do so again until
    you finish a short or long rest.

    """

    name = "Unbreakable Majesty"
    source = "Bard (College of Glamour)"


# College of Swords
class SwordsProficiency(Feature):
    """When you join the College of Swords at 3rd level, you gain proficiency with
    medium armor and the scimitar. If you‘re proficient with a simple or
    martial melee weapon, you can use it as a spellcasting focus for your hard
    spells

    """

    name = "Bonus Proficiencies"
    source = "Bard (College of Swords)"


class BardFightingStyle(FeatureSelector):
    """
    Select a Fighting Style by choosing in feature_choices:

    dueling

    two-weapon fighting
    """

    options = {
        "dueling": Dueling,
        "two-weapon fighting": TwoWeaponFighting,
        "two-weapon": TwoWeaponFighting,
        "dual wield": TwoWeaponFighting,
    }
    name = "Fighting Style (Select One)"
    source = "Bard (College of Swords)"


class BladeFlourish(Feature):
    """At 3rd level, you learn to perform impressive displays of martial prowess
    and speed. Whenever you take the Attack action on your turn, your walking
    speed increases by 10 feet until the end of the turn, and if a weapon
    attack that you make as part of this action hits a creature, you can use
    one of the following Blade Flourish options of your choice. You can use
    only one Blade Flourish option per turn.

    **Defensive Flourish**: You can expend one use of your Bardic Inspiration
    to cause the weapon to deal extra damage to the target you hit. The damage
    equals the number you roll on the Bardic Inspiration die. You also add the
    number rolled to your AC until the start ofyour next turn.

    **Slashing Flourish**: You can expend one use ofyour Bardic Inspiration to
    cause the weapon to deal extra damage to the target you hit and to any
    other creature ofyour choice that you can see within 5 feet ofyou. The
    damage equalsthe number you roll on the Bardic Inspi- ration die.

    **Mobile Flourish**: You can expend one use ofyour Bar- dic InSpiration to
    cause the weapon to deal extra dam- age to the target you hit. The damage
    equals the number you roll on the Bardic Inspiration die. You can also push
    the target up to 5 feet away from you, plus a number of feet equal to the
    number you roll on that die. You can then immediately use your reaction to
    move up to your walking speed to an unoccupied space within 5 feet of the
    target.

    """

    name = "Blade Flourish"
    source = "Bard (College of Swords)"


class MastersFlourish(Feature):
    """Starting at 14th level, whenever you use a Blade Flourish option, you can
    roll a d6 and use it instead of expend- ing a Bardic Inspiration die.
    """

    name = "Master's Flourish"
    source = "Bard (College of Swords)"


# College of Whispers
class PsychicBlades(Feature):
    """When you join the College of Whispers at 3rd level, you gain the
    ability to make your weapon attacks magically toxic to a
    creature's mind. When you hit a creature with a weapon attack, you
    can expend one use ofyour Bardic Inspiration to deal an extra 2d6
    psychic damage to that target. You can do so only once per round
    on your turn. The psychic damage increases when you reach certain
    levels in this class, increasing to 3d6 at 5th level, 5d6 at 10th
    level, and 8d6 at 15th level.

    """

    _name = "Psychic Blades"
    source = "Bard (College of Whispers)"

    @property
    def name(self):
        level = self.owner.Bard.level
        dice = " (2d6)"
        if level >= 5:
            dice = " (3d6)"
        if level >= 10:
            dice = " (5d6)"
        if level >= 15:
            dice = " (8d6)"
        return self._name + dice


class WordsOfTerror(Feature):
    """At 3rd level, you learn to infuse innocent-seeming words with
    an insidious magic that can inspire terror. If you speak to a humanoid alone
    for at least 1 minute, you can attempt to seed paranoia in its mind. At
    the end of the conversation, the target must succeed on a Wisdom saving
    throw against your spell save DC or be frightened of you or another creature
    of your choice. The target is frightened in this way for 1 hour, until it is
    attacked or damaged, or until it witnesses its allies being attacked or
    damaged.

    If the target succeeds on its saving throw, the target has no hint that you
    tried to frighten it. Once you use this feature, you can't use it again
    until you finish a short or long rest.

    """

    name = "Words of Terror"
    source = "Bard (College of Whispers)"


class MantleOfWhispers(Feature):
    """At 6th level, you gain the ability to adopt a humanoid's persona. When a
    humanoid dies within 30 feet of you, you can magically capture its shadow
    using your reac- tion. You retain this shadow until you use it or you
    finish a long rest. You can use the shadow as an action. When you do so, it
    vanishes, magically transforming into a disguise that appears on you. You
    now look like the dead person, but healthy and alive.This disguise lasts
    for 1 hour or until you end it as a bonus action.

    While you're in the disguise, you gain access to all information that the
    humanoid would freely share with a casual acquaintance. Such information
    includes general details on its background and personal life, but doesn't
    include secrets. The information is enough that you can pass yourself off
    as the person by drawing on its memories. Another creature can see through
    this disguise by succeeding on a Wisdom (Insight) check contested by your
    Charisma (Deception) check. You gain a +5 bonus to your check. Once you
    capture a shadow with this feature, you can't capture another one with it
    until you finish a short or long rest.

    """

    name = "Mantle of Whispers"
    source = "Bard (College of Whispers)"


class ShadowLore(Feature):
    """At 14th level, you gain the ability to weave dark magic into your words and
    tap into a creature's deepest fears. As an action, you magically whisper a
    phrase that only one creature ofyour choice within 30 feet of you can
    hear. The target must make a Wisdom saving throw against your spell save
    DC. It automatically succeeds if it doesn't share a language with you or if
    it can't hear you. On a successful saving throw, your whisper sounds like
    unintelligible mumbling and has no effect.

    On a failed saving throw, the target is charmed by you for the next 8 hours
    or until you or your allies attack it, damage it, or force it to make a
    saving throw. It interprets the whispers as a description of its most
    mortifying secret. You gain no knowledge of this secret, but the target
    is convinced you know it. The charmed creature obeys your commands for fear
    that you will reveal its secret. It won't risk its life for you or fight
    for you, unless it was already inclined to do so. It grants you favors and
    gifts it would offer to a close friend. When the effect ends, the creature
    has no understanding of why it held you in such fear. Once you use this
    feature, you can't use it again until / you finish a long rest.

    """

    name = "Shadow Lore"
    source = "Bard (College of Whispers)"



from dungeonsheets.features.features import Feature


class ShelterOfTheFaithful(Feature):
    """As an acolyte, you command the respect of those who share your faith, and
    you can perform the religious ceremonies of your deity. You and your
    adventuring companions can expect to receive free healing and care at a
    temple, shrine, or other established presence of your faith, though you
    must provide any material components needed for spells. Those who share
    your religion will support you (but only you) at a modest lifestyle.

    You might also have ties to a specific temple dedicated to your chosen
    deity or pantheon, and you have a residence there. This could be the temple
    where you used to serve, if you remain on good terms with it, or a temple
    where you have found a new home. While near your temple, you can call upon
    the priests for assistance, provided the assistance you ask for is not
    hazardous and you remain in good standing with your temple.

    """

    name = "Shelter of the Faithful"
    source = "Background (Acolyte)"


class FalseIdentity(Feature):
    """You have created a second identity that includes documentation, established
    acquaintances, and disguises that allow you to assume that
    persona. Additionally, you can forge documents including official papers
    and personal letters, as long as you have seen an example of the kind of
    document or the handwriting you are trying to copy.

    """

    name = "False Identity"
    source = "Background (Charlattan)"


class CriminalContact(Feature):
    """You have a reliable and trustworthy contact who acts as your liaison to a
    network of other criminals. You know how to get messages to and from your
    contact, even over great distances; specifically, you know the local
    messengers, corrupt caravan masters, and seedy sailors who can deliver
    messages for you.

    """

    name = "Criminal Contact"
    source = "Background (Criminal)"


class ByPopularDemand(Feature):
    """You can always find a place to perform, usually in an inn or tavern but
    possibly with a circus, at a theater, or even in a noble's court. At such a
    place, you receive free lodging and food of a modest or comfortable
    standard (depending on the quality of the establishment), as long as you
    perform each night. In addition, your performance makes you something of a
    local figure. When strangers recognize you in a town where you have
    performed, they typically take a liking to you.

    """

    name = "By Popular Demand"
    source = "Background (Entertainer)"


class RusticHospitality(Feature):
    """Since you come from the ranks of the common folk, you fit in among them
    with ease. You can find a place to hide, rest, or recuperate among other
    commoners, unless you have shown yourself to be a danger to them. They will
    shield you from the law or anyone else searching for you, though they will
    not risk their lives for you.

    """

    name = "Rustic Hospitality"
    source = "Background (Folk Hero)"


class GuildMembership(Feature):
    """As an established and respected member of a guild, you can rely on certain
    benefits that membership provides. Your fellow guild members will provide
    you with lodging and food if necessary, and pay for your funeral if
    needed. In some cities and towns, a guildhall offers a central place to
    meet other members of your profession, which can be a good place to meet
    potential patrons, allies, or hirelings.

    Guilds often wield tremendous political power. If you are accused of a
    crime, your guild will support you if a good case can be made for your
    innocence or the crime is justifiable. You can also gain access to powerful
    political figures through the guild, if you are a member in good
    standing. Such connections might require the donation of money or magic
    items to the guild's coffers.

    You must pay dues of 5 gp per month to the guild. If you miss payments, you
    must make up back dues to remain in the guild's good graces.

    """

    name = "Guild Membership"
    source = "Background (Guild Artisan)"


class Discovery(Feature):
    """The quiet seclusion of your extended hermitage gave you access to a unique
    and powerful discovery. The exact nature of this revelation depends on the
    nature of your seclusion. It might be a great truth about the cosmos, the
    deities, the powerful beings of the outer planes, or the forces of
    nature. It could be a site that no one else has ever seen. You might have
    uncovered a fact that has long been forgotten, or unearthed some relic of
    the past that could rewrite history. It might be information that would be
    damaging to the people who or consigned you to exile, and hence the reason
    for your return to society.

    Work with your DM to determine the details of
    your discovery and its impact on the campaign.

    """

    name = "Discovery"
    source = "Background (Hermit)"


class PositionOfPrivilege(Feature):
    """Thanks to your noble birth, people are inclined to think the best of
    you. You are welcome in high society, and people assume you have the right
    to be wherever you are. The common folk make every effort to accommodate
    you and avoid your displeasure, and other people of high birth treat you as
    a member of the same social sphere. You can secure an audience with a local
    noble if you need to.

    """

    name = "Position of Privilege"
    source = "Background (Noble)"


class Wanderer(Feature):
    """You have an excellent memory for maps and geography, and you can always
    recall the general layout of terrain, settlements, and other features
    around you. In addition, you can find food and fresh water for yourself and
    up to five other people each day, provided that the land offers berries,
    small game, water, and so forth.

    """

    name = "Wanderer"
    source = "Background (Outlander)"


class Researcher(Feature):
    """When you attempt to learn or recall a piece of lore, if you do not know
    that information, you often know where and from whom you can obtain
    it. Usually, this information comes from a library, scriptorium,
    university, or a sage or other learned person or creature. Your DM might
    rule that the knowledge you seek is secreted away in an almost inaccessible
    place, or that it simply cannot be found. Unearthing the deepest secrets of
    the multiverse can require an adventure or even a whole campaign.
    """

    name = "Researcher"
    source = "Background (Sage)"


class ShipsPassage(Feature):
    """When you need to, you can secure free passage on a sailing ship for
    yourself and your adventuring companions. You might sail on the ship you
    served on, or another ship you have good relations with (perhaps one
    captained by a former crewmate). Because you're calling in a favor, you
    can't be certain of a schedule or route that will meet your every
    need. Your Dungeon Master will determine how long it takes to get where you
    need to go. In return for your free passage, you and your companions are
    expected to assist the crew during the voyage

    """

    name = "Ship's Passage"
    source = "Background (Sailor)"


class MilitaryRank(Feature):
    """You have a military rank from your career as a soldier. Soldiers loyal to
    your former military organization still recognize your authority and
    influence, and they defer to you if they are of a lower rank. You can
    invoke your rank to exert influence over other soldiers and requisition
    simple equipment or horses for temporary use. You can also usually gain
    access to friendly military encampments and fortresses where your rank is
    recognized.

    """

    name = "Military Rank"
    source = "Background (Soldier)"


class CitySecrets(Feature):
    """You know the secret patterns and flow to cities and can find passages
    through the urban sprawl that others would miss. When you are not in
    combat, you (and companions you lead) can travel between any two locations
    in the city twice as fast as your speed would normally allow.

    """

    name = "City Secrets"
    source = "Background (Urchin)"


# Swords Coast Adventurer's Guide
class AllEyesOnYou(Feature):
    """Your accent, mannerisms, figures of speech, and perhaps even your
    appearance all mark you as foreign. Curious glances are directed your way
    wherever you go, which can be a nuisance, but you also gain the friendly
    interest of scholars and others intrigued by far-off lands, to say nothing
    of everyday folk who are eager to hear stories of your homeland.

    You can parley this attention into access to people and places you might
    not otherwise have, for you and your traveling companions. Noble lords,
    scholars, and merchant princes, to name a few, might be interested in
    hearing about your distant homeland and people.

    """

    name = "All Eyes on You"
    source = "Background (Far Traveler)"


class EarToTheGround(Feature):
    """You are in frequent contact with people in the segment of society that your
    chosen quarries move through. These people might be associated with the
    criminal underworld, the rough-and-tumble folk of the streets, or members
    of high society. This connection comes in the form of a contact in any city
    you visit, a person who provides information about the people and places of
    the local area.

    """

    name = "Ear to the Ground"
    source = "Background (Urban Bounty Hunter)"


class WatchersEye(Feature):
    """Your experience in enforcing the law, and dealing with lawbreakers, gives
    you a feel for local laws and criminals. You can easily find the local
    outpost of the watch or a simila r organization, and just as easily pick
    out the dens of criminal activity in a community, although you're more
    likely to be welcome in the former locations rather than the latter.

    """

    name = "Watcher's Eye"
    source = "Background (City Watch)"


class RespectOfTheStoutFolk(Feature):
    """As well respected as clan crafters are among outsiders, no one esteems them
    quite so highly as dwarves do. You always have free room and board in any
    place where shield dwarves or gold dwarves dwell, and the individuals in
    such a settlement might vie among themselves to determine who can offer you
    (and possibly your compatriots) the finest accommodations and assistance.

    """

    name = "Respect of the Stout Folk"
    source = "Background (Clan Crafter)"


class LibraryAccess(Feature):
    """Though others must often endure extensive interviews and significant fees to
    gain access to even the most common archives in your library, you have free
    and easy access to the majority of the library, though it might also have
    repositories of lore that are too valuable, magical, or secret to permit
    anyone immediate access.

    You have a working knowledge of your cloister's personnel and bureaucracy,
    and you know how to navigate those connections with some ease.

    Additionally, you are likely to gain preferential treatment at other
    libraries across the Realms, as professional courtesy shown to a fellow
    scholar.

    """

    name = "Library Access"
    source = "Background (Cloistered Scholar)"


class CourtFunctionary(Feature):
    """Your knowledge of how bureaucracies function lets you gain access to the
    records and inner workings of any noble court or government you
    encounter. You know who the movers and shakers are, whom to go to for the
    favors you seek, and what the current intrigues of interest in the group
    are.

    """

    name = "Court Functionary"
    source = "Background (Courtier)"


class SafeHaven(Feature):
    """As a faction agent, you have access to a secret network of supporters and
    operatives who can provide assistance on your adventures. You know a set
    of secret signs and passwords you can use to identify such operatives , who
    can provide you with access to a hidden safe house, free room and board, or
    assistance in finding information. These agents never risk their lives
    for you or risk revealing their true identities.

    """

    name = "Save Haven"
    source = "Background (Faction Agent)"


class Inheritance(Feature):
    """Choose or randomly determine your inheritance from among the possibilities
    in the table in SCAG. Work with your Dungeon Master to come up with
    details: Why is your inheritance so important, and what is its full story?
    You might prefer for the DM to invent these details as part of the game,
    allowing you to learn more about your inheritance as your character does.

    The Dungeon Master is free to use your inheritance as a story hook, sending
    you on quests to learn more about its history or true nature, or
    confronting you with foes who want to claim it for themselves or prevent
    you from learning what you seek. The DM also determines the properties of
    your inheritance and how they figure into the item's history and
    importance. For instance, the object might be a minor magic item, or one
    that begins with a modest ability and increases in potency with the passage
    of time. Or, the true nature of your inheritance might not be apparent at
    first and is revealed only when certain conditions are met.

    When you begin your adventuring career, you can decide whether to tell your
    companions about your inheritance right away. Rather than attracting
    attention to yourself, you might want to keep your inheritance a secret
    until you learn more about what it means to you and what it can do for you.

    """

    name = "Inheritance"
    source = "Background (Inheritor)"


class InsideInformant(Feature):
    """You have connections to your previous employer or other groups you
    dealt with during your previous employment. You can communicate
    with your contacts, gaining information at the DM's discretion.

    """

    name = "Inside Informant"
    source = "Background (Rival Intern)"


class KnightlyRegard(Feature):
    """You receive shelter and succor from members of your knightly order and those
    who are sympathetic to its aims. If your order is a religious one, you can
    gain aid from temples and other religious communities of your
    deity. Knights of civic orders can get help from the community- whether a
    lone settlement or a great nation- that they serve, and knights of
    philosophical orders can find help from those they have aided in pursuit of
    their ideals , and those who share those ideals.

    This help comes in the form of shelter and meals, and healing when
    appropriate, as well as occasionally risky assistance, such as a band of
    local citizens rallying to aid a sorely pressed knight in a fight , or
    those who support the order helping to smuggle a knight out of town when
    he or she is being hunted unjustly.

    """

    name = "Knightly Regard"
    source = "Background (Knight of the Order)"


class MercenaryLife(Feature):
    """You know the mercenary life as only someone who has experienced it can. You
    are able to identify mercenary companies by their emblems, and you know a
    little about any such company, including the names and reputations of its
    commanders and leaders, and who has hired them recently. You can find the
    taverns and festhalls where mercenaries abide in any area, as long as you
    speak the language. You can find mercenary work between adventures
    sufficient to maintain a comfortable lifestyle (see "Practicing a
    Profession" under "Downtime Activities" in chapter 8 of the Player's
    Handbook).

    """

    name = "Mercenary Life"
    source = "Background (Mercenary Veteran)"


class UthgardtHeritage(Feature):
    """You have an excellent knowledge of not only your tribe's territory, but also
    the terrain and natural resources of the rest of the North. You are
    familiar enough with any wilderness area that you find twice as much food
    and water as you normally would when you forage there.

    Additionally, you can call upon the hospitality of your people, and those
    folk allied with your tribe, often including members of druid circles,
    tribes of nomadic elves, the Harpers, and the priesthoods devoted to the
    gods of the First Circle.

    """

    name = "Uthgardt Heritage"
    source = "Background (Uthgardt Tribe Member)"


class KeptInStyle(Feature):
    """While you are in Waterdeep or elsewhere in the North your house sees to
    your everyday needs. Your name and signet are sufficient to cover most of
    your expenses; the inns, taverns, and festhalls you frequent are glad to
    record your debt and send an accounting to your family's estate in Waterdeep
    to settle what you owe.

    This advantage enables you to live a comfortable lifestyle without having
    to pay 2 gp a day for it, or reduces the cost of a wealthy or aristocratic
    lifestyle by that amount. You may not maintain a less affluent lifestyle
    and use the difference as income-the benefit is a line of credit, not an
    actual monetary reward.

    """

    name = "Kept in Style"
    source = "Background (Waterdhavian Noble)"



from collections import defaultdict

from dungeonsheets import features, spells, weapons
from dungeonsheets.classes.classes import CharClass, SubClass


class PaladinOath(SubClass):
    name = "Generic Paladin Oath"
    _oath_spells = {3: [], 5: [], 9: [], 13: [], 17: []}

    @property
    def level(self):
        return self.owner.Paladin.level

    @property
    def spells_known(self):
        spells = []
        for lvl, sps in self._oath_spells.items():
            if self.level >= lvl:
                spells.extend(sps)
        return spells

    # All Oath spells are both known and prepared
    @property
    def spells_prepared(self):
        return self.spells_known


class OathOfDevotion(PaladinOath):
    """The Oath of Devotion binds a paladin to the loftiest ideals of justice,
    virtue, and order. Sometim es called cavaliers, white knights, or holy
    warriors, these paladins meet the ideal of the knight in shining armor,
    acting with honor in pursuit o f justice and the greater good. They hold
    themselves to the highest standards of conduct, and some, for better or
    worse, hold the rest of the world to the same standards. Many who swear
    this oath are devoted to gods of law and good and use their gods' tenets as
    the measure o f their devotion. They hold angels-the perfect servants o f
    good-as their ideals, and incorporate images of angelic wings into their
    helmets or coats of arms.

    **Tenets of Devotion**: Though the exact words and strictures of the Oath
    of Devotion vary, paladins of this oath share these tenets.

    --Honesty. Don't lie or cheat. Let your word be your promise.

    --Courage. Never fear to act, though caution is wise.

    --Compassion. Aid others, protect the weak, and punish those who threaten
    them. Show mercy to your foes, but temper it with wisdom.

    --Honor. Treat others with fairness, and let your honorable deeds be an
    example to them. Do as much good as possible while causing the least amount
    of harm.

    --Duty. Be responsible for your actions and their consequences, protect
    those entrusted to your care, and obey those who have just authority over
    you.

    """

    name = "Oath of Devotion"
    _oath_spells = {
        3: [spells.ProtectionFromEvilAndGood, spells.Sanctuary],
        5: [spells.LesserRestoration, spells.ZoneOfTruth],
        9: [spells.BeaconOfHope, spells.DispelMagic],
        13: [spells.FreedomOfMovement, spells.GuardianOfFaith],
        17: [spells.Commune, spells.FlameStrike],
    }
    features_by_level = defaultdict(list)
    features_by_level[3] = [features.SacredWeapon, features.TurnTheUnholy]
    features_by_level[7] = [features.AuraOfDevotion]
    features_by_level[15] = [features.PurityOfSpirit]
    features_by_level[20] = [features.HolyNimbus]


class OathOfAncients(PaladinOath):
    """The Oath of the Ancients is as old as the race of elves and the rituals of
    the druids. Sometimes called fey knights, green knights, or horned knights,
    paladins who swear this oath cast their lot with the side of the light in
    the cosm ic struggle against darkness because they love the beautiful and
    life-giving things of the world, not necessarily because they believe in
    principles of honor, courage, and justice. They adorn their armor and
    clothing with images of growing things-leaves, antlers, or flowers-to
    reflect their commitment to preserving life and light in the world.

    **Tenets of the Ancients**: The tenets of the Oath of the Ancients have
      been preserved for uncounted centuries. This oath emphasizes the
      principles of good above any concerns of law or chaos. Its four central
      principles are simple.

    --Kindle the Light. Through your acts of mercy, kindness, and forgiveness,
    kindle the light of hope in the world, beating back despair.

    --Shelter the Light. Where there is good, beauty, love, and laughter in the
    world, stand against the w ickedness that would swallow it. Where life
    flourishes, stand against the forces that would render it barren.

    --Preserve Your Own Light. Delight in song and laughter, in beauty and
    art. If you allow the light to die in your own heart, you can't preserve it
    in the world.

    --Be the Light. Be a glorious beacon for all who live in despair. Let the
    light of your joy and courage shine forth in all your deeds.

    """

    name = "Oath of The Ancients"
    _oath_spells = {
        3: [spells.EnsnaringStrike, spells.SpeakWithAnimals],
        5: [spells.Moonbeam, spells.MistyStep],
        9: [spells.PlantGrowth, spells.ProtectionFromEnergy],
        13: [spells.IceStorm, spells.Stoneskin],
        17: [spells.CommuneWithNature, spells.TreeStride],
    }
    features_by_level = defaultdict(list)


class OathOfVengeance(PaladinOath):
    """The Oath of Vengeance is a solemn commitment to punish those who have
    committed a grievous sin. When evil forces slaughter helpless villagers,
    when an entire people turns against the will of the gods, when a thieves'
    guild grows too violent and powerful, when a dragon rampages through the
    countryside - at times like these, paladins arise and swear an Oath of
    Vengeance to set right that which has gone wrong. To these paladins -
    sometimes called avengers or dark knights - their own purity is not as
    important as delivering justice.

    **Tenets of Vengeance**: The tenets of the Oath of Vengeance vary by
    paladin, but all the tenets revolve around punishing wrongdoers by any
    means necessary. Paladins who uphold these tenets are willing to sacrifice
    even their own righteousness to mete out justice upon those who do evil, so
    the paladins are often neutral or lawful neutral in alignment. The core
    principles of the tenets are brutally simple.

    --Fight the Greater Evil. Faced with a choice of fighting my sworn foes or
    combating a lesser evil. I choose the greater evil.

    --No Mercy for the Wicked. Ordinary foes might win my mercy, but my sworn
    enemies do not.

    --By Any Means Necessary. My qualms can't get in the way of exterminating
    my foes.

    --Restitution. If my foes wreak ruin on the world, it is because I failed
    to stop them. I must help those harmed by their misdeeds.

    """

    name = "Oath of Vengeance"
    _oath_spells = {
        3: [spells.Bane, spells.HuntersMark],
        5: [spells.HoldPerson, spells.MistyStep],
        9: [spells.Haste, spells.ProtectionFromEnergy],
        13: [spells.Banishment, spells.DimensionDoor],
        17: [spells.HoldMonster, spells.Scrying],
    }
    features_by_level = defaultdict(list)


class OathOfCrown(PaladinOath):
    """The Oath of the Crown is sworn to the ideals of civilization, be it the
    spirit of a nation, fealty to a sovereign, or service to a deity of law and
    rulership. The paladins who swear this oath dedicate themselves to serving
    society and, in particular, the just laws that hold society together. These
    paladins are the watchful guardians on the walls, standing against the
    chaotic tides of barbarism that threaten to tear down all that
    civilization has built, and are commonly known as guardians, exemplars,
    or sentinels. Often, paladins who swear this oath are members of an order
    of knighthood in service to a nation or a sovereign, and undergo their oath
    as part of their admission to the order's ranks.

    **Tenets of the Crown**: The tenets of the Oath of the Crown are often set
    by the sovereign to which their oath is sworn, but generally emphasize
    the following tenets.

    --Law. The law is paramount. It is the mortar that holds the stones of
    civilization together, and it must be respected.

    --Loyalty. Your word is your bond. Without loyalty, oaths and laws are
    meaningless.

    --Courage. You must be willing to do what needs to be done for the sake of
    order, even in the face of overwhelming odds. If you don't act, then who
    will?

    --Responsibility. You must deal with the consequences of your actions, and
    you are responsible for fulfilling your duties and obligations.

    """

    name = "Oath of The Crown"
    _oath_spells = {
        3: [spells.Command, spells.CompelledDuel],
        5: [spells.WardingBond, spells.ZoneOfTruth],
        9: [spells.AuraOfVitality, spells.SpiritGuardians],
        13: [spells.Banishment, spells.GuardianOfFaith],
        17: [spells.CircleOfPower, spells.Geas],
    }
    features_by_level = defaultdict(list)


class OathOfConquest(PaladinOath):
    """The Oath of Conquest calls to paladins who seek glory in battle and
    the subjugation of their enemies. It isn't enough for these
    paladins to establish order. They must crush the forces of
    chaos. Sometimes called knight tyrants or iron mongers, those
    who swear this oath gather into grim orders that serve gods or
    philosophies of war and well-ordered might.

    Some of these paladins go so far as to consort with the powers of
    the Nine Hells, valuing the rule of law over the balm of
    mercy. The archdevil Bel, warlord of Avernus, counts many of these
    paladins--called hell knights--as his most ardent supporters. Hell
    knights cover their armor with trophies taken from fallen en-
    emies, a grim warning to any who dare oppose them and the decrees
    of their lords. These knights are often most fiercely resisted by
    other paladins of this oath, who believe that the hell knights
    have wandered too far into darkness.

    **Tenets of Conquest:** A paladin who takes this oath has the
    tenets of conquest seared on the upper arm.

    Douse the Flame of Hope.
      It is not enough to merely defeat an enemy in battle. Your
      victory must be so overwhelming that your enemies' will to
      fight is shattered forever. A blade can end a life. Fear can end
      an empire.
    Rule with an Iron Fist.
      Once you have conquered, tolerate no dissent. Your word is
      law. Those who obey it shall be favored. Those who defy it shall
      be punished as an example to all who might follow.
    Strength Above All.
      You shall rule until a stronger one arises. Then you must grow
      mightier and meet the challenge, or fall to your own ruin.

    """

    name = "Oath of Conquest"
    _oath_spells = {
        3: [spells.ArmorOfAgathys, spells.Command],
        5: [spells.HoldPerson, spells.SpiritualWeapon],
        9: [spells.BestowCurse, spells.Fear],
        13: [spells.DominateBeast, spells.Stoneskin],
        17: [spells.Cloudkill, spells.DominatePerson],
    }
    features_by_level = defaultdict(list)


class OathOfRedemption(PaladinOath):
    """The Oath of Redemption sets a paladin on a difficult path, one that requires
    a holy warrior to use violence only as a last resort. Paladins who dedicate
    themselves to this oath believe that any person can be redeemed and that
    the path of benevolence and justice is one that anyone can walk. These
    paladins face evil creatures in the hope of turning their foes to the
    light, and they slay their enemies only when such a deed will clearly save
    other lives. Paladins who follow this path are known as redeemers.

    While redeemers are idealists, they are no fools. Re- deemers know that
    undead, demons, devils, and other supernatural threats can be inherently
    evil. Against such fees, paladins who swear this oath bring the full wrath
    of their weapons and spells to bear. Yet the re- deemers still pray that,
    one day, even creatures of wick- edness will invite their own redemption.

    **Tenets of Redemption**: The tenets of the Oath of Redemption hold a
    paladin to a high standard of peace and justice.

    --Peace. Violence is a weapon of last resort. Diplomacy and understanding
    are the paths to long-lasting peace.

    --Innocence. All people begin life in an innocent state, and it is their
    environment or the influence of dark forces that drives them to evil. By
    setting the proper example, and working to heal the wounds of a deeply
    flawed world, you can set anyone on a righteous path.

    --Patience. Change takes time. Those who have walked the path of the wicked
    must be given reminders to keep them honest and true. Once you have planted
    the seed of righteousness in a creature, you must work day after day to
    allow that seed to survive and flourish.

    --Wisdom. Your heart and mind must stay clear, for eventually you will be
    forced to admit defeat. While ev- ery creature can be redeemed, some are so
    far along the path of evil that you have no choice but to end their lives
    for the greater good. Any such action must be carefully weighed and the
    consequences fully understood, but once you have made the decision, follow
    through with it knowing your path is just.

    """

    name = "Oath of Redemption"
    _oath_spells = {
        3: [spells.Sanctuary, spells.Sleep],
        5: [spells.CalmEmotions, spells.HoldPerson],
        9: [spells.Counterspell, spells.HypnoticPattern],
        13: [spells.OtilukesResilientSphere, spells.Stoneskin],
        17: [spells.HoldMonster, spells.WallOfForce],
    }
    features_by_level = defaultdict(list)
    features_by_level[3] = [features.EmissaryOfPeace, features.RebukeTheViolent]
    features_by_level[7] = [features.AuraOfTheGuardian]
    features_by_level[15] = [features.ProtectiveSpirit]
    features_by_level[20] = [features.EmissaryOfRedemption]


# Custom
class OathOfZor(PaladinOath):
    """The Oath of Zor

    **Tenets of Zor**:

    --Courage. Never fear to act, though caution is wise.

    --Honesty. Don't lie or cheat. Let your word be your promise.

    --Innocence. All people begin life in an innocent state, and it is their
    environment or the influence of dark forces that drives them to evil. By
    setting the proper example, and working to heal the wounds of a deeply
    flawed world, you can set anyone on a righteous path.

    --Restitution. If my foes wreak ruin on the world, it is because I failed
    to stop them. I must help those harmed by their misdeeds.

    --Patience. Change takes time. Those who have walked the path of the wicked
    must be given reminders to keep them honest and true. Once you have planted
    the seed of righteousness in a creature, you must work day after day to
    allow that seed to survive and flourish.


    """

    name = "Oath of Zor"
    _oath_spells = {
        3: [spells.Sanctuary, spells.Sleep],
        5: [spells.CalmEmotions, spells.HoldPerson],
        9: [spells.Counterspell, spells.HypnoticPattern],
        13: [spells.OtilukesResilientSphere, spells.Stoneskin],
        17: [spells.HoldMonster, spells.WallOfForce],
    }
    features_by_level = defaultdict(list)
    features_by_level[3] = [features.EmissaryOfPeace, features.RebukeTheViolent]
    features_by_level[7] = [features.AuraOfTheGuardian]
    features_by_level[15] = [features.ProtectiveSpirit]
    features_by_level[20] = [features.EmissaryOfRedemption]


class Paladin(CharClass):
    name = "Paladin"
    hit_dice_faces = 10
    subclass_select_level = 3
    saving_throw_proficiencies = ("wisdom", "charisma")
    primary_abilities = ("strength", "charisma")
    _proficiencies_text = ("All armor", "shields", "simple weapons", "martial weapons")
    weapon_proficiencies = (weapons.SimpleWeapon, weapons.MartialWeapon)
    multiclass_weapon_proficiencies = weapon_proficiencies
    _multiclass_proficiencies_text = (
        "light armor",
        "medium armor",
        "shields",
        "simple weapons",
        "martial weapons",
    )
    class_skill_choices = (
        "Athletics",
        "Insight",
        "Intimidation",
        "Medicine",
        "Persuasion",
        "Religion",
    )
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.DivineSense, features.LayOnHands]
    features_by_level[2] = [features.PaladinFightingStyle, features.DivineSmite]
    features_by_level[3] = [features.DivineHealth, features.ChannelDivinityPaladin]
    features_by_level[5] = [features.ExtraAttackPaladin]
    features_by_level[6] = [features.AuraOfProtection]
    features_by_level[10] = [features.AuraOfCourage]
    features_by_level[11] = [features.ImprovedDivineSmite]
    features_by_level[14] = [features.CleansingTouch]
    subclasses_available = (
        OathOfDevotion,
        OathOfAncients,
        OathOfVengeance,
        OathOfCrown,
        OathOfConquest,
        OathOfRedemption,
        OathOfZor,
    )
    spellcasting_ability = "charisma"
    spell_slots_by_level = {
        # char_lvl: (cantrips, 1st, 2nd, 3rd, ...)
        1: (0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
        2: (0, 2, 0, 0, 0, 0, 0, 0, 0, 0),
        3: (0, 3, 0, 0, 0, 0, 0, 0, 0, 0),
        4: (0, 3, 0, 0, 0, 0, 0, 0, 0, 0),
        5: (0, 4, 2, 0, 0, 0, 0, 0, 0, 0),
        6: (0, 4, 2, 0, 0, 0, 0, 0, 0, 0),
        7: (0, 4, 3, 0, 0, 0, 0, 0, 0, 0),
        8: (0, 4, 3, 0, 0, 0, 0, 0, 0, 0),
        9: (0, 4, 3, 2, 0, 0, 0, 0, 0, 0),
        10: (0, 4, 3, 2, 0, 0, 0, 0, 0, 0),
        11: (0, 4, 3, 3, 0, 0, 0, 0, 0, 0),
        12: (0, 4, 3, 3, 0, 0, 0, 0, 0, 0),
        13: (0, 4, 3, 3, 1, 0, 0, 0, 0, 0),
        14: (0, 4, 3, 3, 1, 0, 0, 0, 0, 0),
        15: (0, 4, 3, 3, 2, 0, 0, 0, 0, 0),
        16: (0, 4, 3, 3, 2, 0, 0, 0, 0, 0),
        17: (0, 4, 3, 3, 3, 1, 0, 0, 0, 0),
        18: (0, 4, 3, 3, 3, 1, 0, 0, 0, 0),
        19: (0, 4, 3, 3, 3, 2, 0, 0, 0, 0),
        20: (0, 4, 3, 3, 3, 2, 0, 0, 0, 0),
    }



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class YoungBlackDragon(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one target. Hit: 15
      (2d10 + 4) piercing damage plus 4 (1d8) acid damage.
    Claw.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    Acid Breath.
      The dragon exhales acid in a 30-foot line that is 5 feet wide. Each
      creature in that line must make a DC 14 Dexterity saving throw, taking
      49 (11d8) acid damage on a failed save, or half as much damage on a
      successful one.
    """
    name = "Young Black Dragon"
    description = "Large dragon, chaotic evil"
    challenge_rating = 7
    armor_class = 18
    skills = "Perception +6, Stealth +5"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 16"
    languages = "Common, Draconic"
    strength = Ability(19)
    dexterity = Ability(14)
    constitution = Ability(17)
    intelligence = Ability(12)
    wisdom = Ability(11)
    charisma = Ability(15)
    speed = 40
    swim_speed = 40
    fly_speed = 80
    climb_speed = 0
    hp_max = 127
    hit_dice = "15d10"
    spells = []


class YoungBlueDragon(Monster):
    """
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +9 to hit, reach 10 ft., one target. Hit: 16
      (2d10 + 5) piercing damage plus 5 (1d10) lightning damage.
    Claw.
      Melee Weapon Attack: +9 to hit, reach 5 ft., one target. Hit: 12 (2d6
      + 5) slashing damage.
    Lightning Breath.
      The dragon exhales lightning in an 60-foot line that is 5 feet wide.
      Each creature in that line must make a DC 16 Dexterity saving throw,
      taking 55 (10d10) lightning damage on a failed save, or half as much
      damage on a successful one.
    """
    name = "Young Blue Dragon"
    description = "Large dragon, lawful evil"
    challenge_rating = 9
    armor_class = 18
    skills = "Perception +9, Stealth +4"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 19"
    languages = "Common, Draconic"
    strength = Ability(21)
    dexterity = Ability(10)
    constitution = Ability(19)
    intelligence = Ability(14)
    wisdom = Ability(13)
    charisma = Ability(17)
    speed = 40
    swim_speed = 0
    fly_speed = 80
    climb_speed = 0
    hp_max = 152
    hit_dice = "16d10"
    spells = []


class YoungBrassDragon(Monster):
    """
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one target. Hit: 15
      (2d10 + 4) piercing damage.
    Claw.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Fire Breath. The dragon exhales fire in a 40-foot line that is 5 feet
      wide. Each creature in that line must make a DC 14 Dexterity saving
      throw, taking 42 (12d6) fire damage on a failed save, or half as much
      damage on a successful one.
      
      Sleep Breath. The dragon exhales sleep gas in a 30-foot cone. Each
      creature in that area must succeed on a DC 14 Constitution saving
      throw or fall unconscious for 5 minutes. This effect ends for a
      creature if the creature takes damage or someone uses an action to
      wake it.
    """
    name = "Young Brass Dragon"
    description = "Large dragon, chaotic good"
    challenge_rating = 6
    armor_class = 17
    skills = "Perception +6, Persuasion +5, Stealth +3"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 16"
    languages = "Common, Draconic"
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(17)
    intelligence = Ability(12)
    wisdom = Ability(11)
    charisma = Ability(15)
    speed = 40
    swim_speed = 0
    fly_speed = 80
    climb_speed = 0
    hp_max = 110
    hit_dice = "13d10"
    spells = []


class YoungBronzeDragon(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one target. Hit: 16
      (2d10 + 5) piercing damage.
    Claw.
      Melee Weapon Attack: +8 to hit, reach 5 ft., one target. Hit: 12 (2d6
      + 5) slashing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Lightning Breath. The dragon exhales lightning in a 60-foot line that
      is 5 feet wide. Each creature in that line must make a DC 15 Dexterity
      saving throw, taking 55 (10d10) lightning damage on a failed save, or
      half as much damage on a successful one.
      
      Repulsion Breath. The dragon exhales repulsion energy in a 30-foot
      cone. Each creature in that area must succeed on a DC 15 Strength
      saving throw. On a failed save, the creature is pushed 40 feet away
      from the dragon.
    """
    name = "Young Bronze Dragon"
    description = "Large dragon, lawful good"
    challenge_rating = 8
    armor_class = 18
    skills = "Insight +4, Perception +7, Stealth +3"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 17"
    languages = "Common, Draconic"
    strength = Ability(21)
    dexterity = Ability(10)
    constitution = Ability(19)
    intelligence = Ability(14)
    wisdom = Ability(13)
    charisma = Ability(17)
    speed = 40
    swim_speed = 40
    fly_speed = 80
    climb_speed = 0
    hp_max = 142
    hit_dice = "15d10"
    spells = []


class YoungCopperDragon(Monster):
    """
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one target. Hit: 15
      (2d10 + 4) piercing damage.
    Claw.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Acid Breath. The dragon exhales acid in an 40-foot line that is 5 feet
      wide. Each creature in that line must make a DC 14 Dexterity saving
      throw, taking 40 (9d8) acid damage on a failed save, or half as much
      damage on a successful one.
      
      Slowing Breath. The dragon exhales gas in a 30-foot cone. Each
      creature in that area must succeed on a DC 14 Constitution saving
      throw. On a failed save, the creature can't use reactions, its speed
      is halved, and it can't make more than one attack on its turn. In
      addition, the creature can use either an action or a bonus action on
      its turn, but not both. These effects last for 1 minute. The creature
      can repeat the saving throw at the end of each of its turns, ending
      the effect on itself with a successful save.
    """
    name = "Young Copper Dragon"
    description = "Large dragon, chaotic good"
    challenge_rating = 7
    armor_class = 17
    skills = "Deception +5, Perception +7, Stealth +4"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 17"
    languages = "Common, Draconic"
    strength = Ability(19)
    dexterity = Ability(12)
    constitution = Ability(17)
    intelligence = Ability(16)
    wisdom = Ability(13)
    charisma = Ability(15)
    speed = 40
    swim_speed = 0
    fly_speed = 80
    climb_speed = 40
    hp_max = 119
    hit_dice = "14d10"
    spells = []


class YoungGoldDragon(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +10 to hit, reach 10 ft., one target. Hit: 17
      (2d10 + 6) piercing damage.
    Claw.
      Melee Weapon Attack: +10 to hit, reach 5 ft., one target. Hit: 13 (2d6
      + 6) slashing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Fire Breath. The dragon exhales fire in a 30-foot cone. Each creature
      in that area must make a DC 17 Dexterity saving throw, taking 55
      (10d10) fire damage on a failed save, or half as much damage on a
      successful one.
      
      Weakening Breath. The dragon exhales gas in a 30-foot cone. Each
      creature in that area must succeed on a DC 17 Strength saving throw or
      have disadvantage on Strength-based attack rolls, Strength checks, and
      Strength saving throws for 1 minute. A creature can repeat the saving
      throw at the end of each of its turns, ending the effect on itself on
      a success.
    """
    name = "Young Gold Dragon"
    description = "Large dragon, lawful good"
    challenge_rating = 10
    armor_class = 18
    skills = "Insight +5, Perception +9, Persuasion +9, Stealth +6"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 19"
    languages = "Common, Draconic"
    strength = Ability(23)
    dexterity = Ability(14)
    constitution = Ability(21)
    intelligence = Ability(16)
    wisdom = Ability(13)
    charisma = Ability(20)
    speed = 40
    swim_speed = 40
    fly_speed = 80
    climb_speed = 0
    hp_max = 178
    hit_dice = "17d10"
    spells = []


class YoungGreenDragon(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one target. Hit: 15
      (2d10 + 4) piercing damage plus 7 (2d6) poison damage.
    Claw.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    Poison Breath.
      The dragon exhales poisonous gas in a 30-foot cone. Each creature in
      that area must make a DC 14 Constitution saving throw, taking 42
      (12d6) poison damage on a failed save, or half as much damage on a
      successful one.
    """
    name = "Young Green Dragon"
    description = "Large dragon, lawful evil"
    challenge_rating = 8
    armor_class = 18
    skills = "Deception +5, Perception +7, Stealth +4"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 17"
    languages = "Common, Draconic"
    strength = Ability(19)
    dexterity = Ability(12)
    constitution = Ability(17)
    intelligence = Ability(16)
    wisdom = Ability(13)
    charisma = Ability(15)
    speed = 40
    swim_speed = 40
    fly_speed = 80
    climb_speed = 0
    hp_max = 136
    hit_dice = "16d10"
    spells = []


class YoungRedDragon(Monster):
    """
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +10 to hit, reach 10 ft., one target. Hit: 17
      (2d10 + 6) piercing damage plus 3 (1d6) fire damage.
    Claw.
      Melee Weapon Attack: +10 to hit, reach 5 ft., one target. Hit: 13 (2d6
      + 6) slashing damage.
    Fire Breath.
      The dragon exhales fire in a 30-foot cone. Each creature in that area
      must make a DC 17 Dexterity saving throw, taking 56 (16d6) fire damage
      on a failed save, or half as much damage on a successful one.
    """
    name = "Young Red Dragon"
    description = "Large dragon, chaotic evil"
    challenge_rating = 10
    armor_class = 18
    skills = "Perception +8, Stealth +4"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 18"
    languages = "Common, Draconic"
    strength = Ability(23)
    dexterity = Ability(10)
    constitution = Ability(21)
    intelligence = Ability(14)
    wisdom = Ability(11)
    charisma = Ability(19)
    speed = 40
    swim_speed = 0
    fly_speed = 80
    climb_speed = 40
    hp_max = 178
    hit_dice = "17d10"
    spells = []


class YoungSilverDragon(Monster):
    """
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +10 to hit, reach 10 ft., one target. Hit: 17
      (2d10 + 6) piercing damage.
    Claw.
      Melee Weapon Attack: +10 to hit, reach 5 ft., one target. Hit: 13 (2d6
      + 6) slashing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Cold Breath. The dragon exhales an icy blast in a 30-foot cone. Each
      creature in that area must make a DC 17 Constitution saving throw,
      taking 54 (12d8) cold damage on a failed save, or half as much damage
      on a successful one.
      
      Paralyzing Breath. The dragon exhales paralyzing gas in a 30-foot
      cone. Each creature in that area must succeed on a DC 17 Constitution
      saving throw or be paralyzed for 1 minute. A creature can repeat the
      saving throw at the end of each of its turns, ending the effect on
      itself on a success.
    """
    name = "Young Silver Dragon"
    description = "Large dragon, lawful good"
    challenge_rating = 9
    armor_class = 18
    skills = "Arcana +6, History +6, Perception +8, Stealth +4"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 18"
    languages = "Common, Draconic"
    strength = Ability(23)
    dexterity = Ability(10)
    constitution = Ability(21)
    intelligence = Ability(14)
    wisdom = Ability(11)
    charisma = Ability(19)
    speed = 40
    swim_speed = 0
    fly_speed = 80
    climb_speed = 0
    hp_max = 168
    hit_dice = "16d10"
    spells = []


class YoungWhiteDragon(Monster):
    """
    Ice Walk.
      The dragon can move across and climb icy surfaces without needing to
      make an ability check. Additionally, difficult terrain composed of ice
      or snow doesn't cost it extra moment.
    Multiattack.
      The dragon makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one target. Hit: 15
      (2d10 + 4) piercing damage plus 4 (1d8) cold damage.
    Claw.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    Cold Breath.
      The dragon exhales an icy blast in a 30-foot cone. Each creature in
      that area must make a DC 15 Constitution saving throw, taking 45
      (10d8) cold damage on a failed save, or half as much damage on a
      successful one.
    """
    name = "Young White Dragon"
    description = "Large dragon, chaotic evil"
    challenge_rating = 6
    armor_class = 17
    skills = "Perception +6, Stealth +3"
    senses = "Blindsight 30 ft., Darkvision 120 ft., Passive Perception 16"
    languages = "Common, Draconic"
    strength = Ability(18)
    dexterity = Ability(10)
    constitution = Ability(18)
    intelligence = Ability(6)
    wisdom = Ability(11)
    charisma = Ability(12)
    speed = 40
    swim_speed = 40
    fly_speed = 80
    climb_speed = 0
    hp_max = 133
    hit_dice = "14d10"
    spells = []



#!/usr/bin/env python

import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="dungeonsheets",
    version=read("VERSION"),
    description="Dungeons and Dragons 5e Character Tools",
    long_description=read("README.rst"),
    long_description_content_type="text/x-rst",
    keywords="D&D character sheets",
    author="Mark Wolfman",
    author_email="canismarko@gmail.com",
    license="GPLv3",
    url="https://github.com/canismarko/dungeon-sheets",
    download_url="https://github.com/canismarko/dungeon-sheets/archive/master.zip",
    packages=find_packages(),
    package_data={
        "dungeonsheets": [
            "forms/*pdf",
            "forms/*.tex",
            "forms/*.txt",
            "modules/DND-5e-LaTeX-Template/*",
            "modules/DND-5e-LaTeX-Template/lib/*",
            "modules/DND-5e-LaTeX-Template/img/*",
            "../VERSION",
        ]
    },
    install_requires=[
        "fdfgen",
        "npyscreen",
        "jinja2",
        "pdfrw",
        "sphinx",
        "EbookLib",
        "reportlab",
    ],
    entry_points={
        "console_scripts": [
            "makesheets = dungeonsheets.make_sheets:main",
            "create-character = dungeonsheets.create_character:main",
        ]
    },
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Games/Entertainment :: Role-Playing",
    ],
)



from dungeonsheets.spells.spells import Spell


class UnseenServant(Spell):
    """This spell creates an invisible, mindless, shapeless force that performs simple
    tasks at your command until the spell ends. The servant springs into existence
    in an unoccupied space on the ground within range. It has AC 10, 1 hit point,
    and a Strength of 2, and it can't attack. If it drops to 0 hit points, the spell
    ends.

    Once on each of your turns as a bonus action, you can mentally command
    the servant to move up to 15 feet and inteact with an object. The servant can
    perform simple tasks that a human servant could do, such as fetching things,
    cleaning, mending, folding clothes, lighting fires, serving food, and pouring
    wine. Once you give the command, the servant performs the task to the best of
    its ability until it completes the task, then waits for your next command.

    If
    you command the servant to perform a task that would move it more than 60 feet
    away from you, the spell ends.
    """

    name = "Unseen Servant"
    level = 1
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = "A piece of string and a bit of wood"
    duration = "1 hour"
    ritual = True
    magic_school = "Conjuration"
    classes = ("Bard", "Warlock", "Wizard")



from dungeonsheets.spells.spells import Spell


class WallOfFire(Spell):
    """You create a wall of fire on a solid surface within range. You can
    make the wall up to 60 feet long, 20 feet high, and 1 foot thick,
    or a ringed wall up to 20 feet in diameter, 20 feet high, and 1
    foot thick. The wall is opaque and lasts for the duration.

    When the wall appears, each creature within its area must make a
    Dexterity saving throw. On a failed save, a creature takes 5d8
    fire damage, or half as much damage on a successful save.

    One side of the wall, selected by you when you cast this spell,
    deals 5d8 fire damage to each creature that ends its turn within
    10 feet of that side or inside the wall. A creature takes the same
    damage when it enters the wall for the first time on a turn or
    ends its turn there. The other side of the wall deals no damage.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 5th level or higher, the damage increases by 1d8 for each slot
    level above 4th.

    """

    name = "Wall Of Fire"
    level = 4
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S", "M")
    materials = "A small piece of phosphorus"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Sorcerer", "Wizard")


class WallOfForce(Spell):
    """An invisible wall of force springs into existence at a point you
    choose within range.  The wall appears in any orientation you
    choose, as a horizontal or vertical barrier or at an angle. It can
    be free floating or resting on a solid surface. You can form it
    into a hemispherical dome or a sphere with a radius of up to 10
    feet, or you can shape a flat surface made up of ten
    10-foot-by-10-foot panels. Each panel must be continguous with
    another panel. In any form, the wall is 1/4 inch thick. It lasts
    for the duration. If the wall cuts through a creature's space when
    it appears, the creature is pushed to one side of the wall (your
    choice which side).

    Nothing can physically pass through the wall. It is immune to all
    damage and can't be dispelled by dispel magic. A disintegrate
    spell destroys the wall instantly, however. The wall also extends
    into the Ethereal Plane, blocking ethereal travel through the
    wall.

    """

    name = "Wall Of Force"
    level = 5
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S", "M")
    materials = "A pinch of powder made by crushing a clear gemstone"
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Evocation"
    classes = ("Wizard",)


class WallOfIce(Spell):
    """You create a wall of ice on a solid surface within range. You can
    form it into a hemispherical dome or a sphere with radius of up to
    10 feet, or you can shape a flat surfcae made up of ten
    10-foot-square panels. Each panel must be contiguous with another
    panel. In any form, the wall is 1 foot thick and lasts for the
    duration.

    If the wall cuts through a creature's space when it appears, the
    creature within its area is pushed to one side of the wall and
    must make a Dexterity saving throw. On a failed save, the creature
    takes 10d6 cold damage, or half as much damage on a successful
    save.

    The wall is an object that can be damaged and thus breached. It
    has AC 12 and 30 hit points per 10-foot section, and it is
    vulnerable to fire damage. Reducing a 10-foot section of wall to 0
    hit points destroys it and leaves behind a sheet of frigid air int
    he space the wall occupied. A creature moving through the sheet of
    frigid air for the first time on a turn must make a Constitution
    saaving throw. The creature takes 5f6 cold damage on a failed
    save, or half as much damage on a successful one.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 7th level or higher, the damage the wall deals when it appears
    increases by 2d6, and the damage from passing through the sheet of
    frigid air increases by 1d6, for each slot level above 6th.

    """

    name = "Wall Of Ice"
    level = 6
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S", "M")
    materials = "A small piece of quartz"
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Evocation"
    classes = ("Wizard",)


class WallOfLight(Spell):
    """A shimmering wall of bright light appears at a point you choose
    within range.  The wall appears in any orientation you choose:
    horizontally, vertically, or diagonally. It can be free floating,
    or it can rest on a solid surface. The wall can be up to 60 feet
    long, 10 feet high, and 5 feet thick. The wall blocks line of
    sight, but creatures and objects can pass through it. It emits
    bright light out to 120 feet and dim light for an additional 120
    feet.

    When the wall appears, each creature in its area must make a
    Constitution saving throw. On a failed save, a creature takes 4d8
    radiant damage, and it is blinded for 1 minute. On a successful
    save, it takes half as much damage and isn't blinded. A blinded
    creature can make a Constitution saving throw at the end of each
    of its turns, ending the effect on itself on a success.

    A creature that ends its turn in the wall's area takes 4d8 radiant
    damage.  Until the spell ends, you can use an action to launch a
    beam of radiance from the wall at one creature you can see within
    60 feet of it. Make a ranged spell attack. On a hit, the target
    takes 4d8 radiant damage. Whether you hit or miss, reduce the
    length of the wall by 10 feet. If the wall's length drops to 0
    feet, the spell ends.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 6th level or higher, the damage increases by 1d8 for each slot
    level above 5th.

    """

    name = "Wall Of Light"
    level = 5
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S", "M")
    materials = "A hand mirror"
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Evocation"
    classes = ("Sorcerer", "Warlock", "Wizard")


class WallOfSand(Spell):
    """You conjure up a wall of swirling sand on the ground at a point you can see
    within range. You can make the wall up to 30 feet long, 10 feet high, and 10
    feet thick, and it vanishes when the spell ends. It blocks line of sight but not
    movement. A creature is blinded while in the wall's space and must spend 3 feet
    of movement for every 1 foot it moves there.
    """

    name = "Wall Of Sand"
    level = 3
    casting_time = "1 action"
    casting_range = "90 feet"
    components = ("V", "S", "M")
    materials = "A handful of sand"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Wizard",)


class WallOfStone(Spell):
    """A nonmagical wall of solid stone springs into existence at a point
    you choose within range.  The wall is 6 inches thick and is
    composed of ten 10-foot- by-10-foot panels. Each panel must be
    contiguous with at least on other panel.  Alternatively, you can
    create 10-foot-by-20-foot panels that are only 3 inches thick.

    If the wall cuts through a creature's space when it appears, the
    creature is pushed to one side of the wall (your choice). If a
    creature would be surrounded on all sides by the wall (or the wall
    and another solid surface), that creature can make a Dexterity
    saving throw. On a success, it can use its reaction to move up to
    its speed so that it is no longer enclosed by the wall.

    The wall can have any shape you desire, though it can't occupy the
    same space as a creature or object. the wall doesn't need to be
    vertical or resting on any firm foundation. It must, however,
    merge with and be solidly supported by existing stone. Thus you
    can use this spell to bridge a chasm or create a ramp.

    If you create a span greater than 20 feet in length, you must
    halve the size of each panel to create supports. You can crudely
    shape the wall to create crenellations, battlements, and so on.

    The wall is an object made of stone that can be damaged and thus
    breached. Each panel has AC 15 and 30 hit points per inch of
    thickness. Reducing a panel to 0 hit points destroys it and might
    cause connected panels to collapse at the DM's discretion.

    If you maintain your concentration on this spell for its whole
    duration, the wall becomes permanent and can't be
    dispelled. Otherwise, the wall disappears when the spell ends.

    """

    name = "Wall Of Stone"
    level = 5
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S", "M")
    materials = "A small block of granite"
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Sorcerer", "Wizard")


class WallOfThorns(Spell):
    """You create a wall of tough, pliable, tangled brush bristling with needle-sharp
    thorns. The wall appears within range on a solid surface and lasts for the
    duration. You choose to make the wall up to 60 feet long, 10 feet high, and 5
    feet thick or a circle that has a 20-foot diameter and is up to 20 feet high and
    5  feet thick. The wall blocks line of sight.

    When the wall appears, each
    creature within its area must make a Dexterity saving throw. On a failed save, a
    creature takes 7d8 piercing damage, or half as much damage on a successful
    save.

    A creature can move through the wall, albeit slowly and painfully. For
    every 1 foot a creature moves through the wall, it must spend 4 feet of
    movement. Furthermore, the first time a creature enters the wall on a turn or
    ends its turn there, the creature must make a Dexterity saving throw. It takes
    7d8 slashing damage on a failed save, or half as much on a successful save.

    At
    Higher Levels: When you cast this spell using a spell slot of 7th level or
    higher, both types o f damage increase by 1d8 for each slot level above 6th.
    """

    name = "Wall Of Thorns"
    level = 6
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S", "M")
    materials = "A handful of thorns"
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Druid",)


class WallOfWater(Spell):
    """(a drop of water)

    You conjure up a wall of water on the ground at a point you can
    see within range. You can make the wall up to 30 feet long, 10
    feet high, and 1 foot thick, or you can make a ringed wall up to
    20 feet in diameter, 20 feet high, and 1 foot thick. The wall
    vanishes when the spell ends. The wall's space is difficult
    terrain.

    Any ranged weapon attack that enters the wall's space has
    disadvantage on the attack roll, and fire damage is halved if the
    fire effect passes through the wall to reach its target. Spells
    that deal cold damage that pass through the wall cause the area of
    the wall they pass through to freeze solid (at least a 5-foot
    square section is frozen). Each 5-foot-square frozen section has
    AC 5 and 15 hit points. Reducing a frozen section to 0 hit points
    destroys it. When a section is destroyed, the wall’s water doesn’t
    fill it.

    """

    name = "Wall Of Water"
    level = 3
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Sorcerer", "Wizard")


class WardingBond(Spell):
    """This spell wards a willing creature you touch and creates a mystic connection
    between you and the target until the spell ends.

    While the target is within 60
    feet of you, it gains a +1 bonus to AC and saving throws, and it has resistance
    to all damage. Also, each time it takes damage, you take the same amount of
    damage.

    The spell ends if you drop to 0 hit points or if you and the target
    become separated by more than 60 feet. It also ends if the spell is cast again
    on either of the connected creatures. You can also dismiss the spell as an
    action.
    """

    name = "Warding Bond"
    level = 2
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = (
        "A pair of platinum rings worth at least 50 gp each, which you and target must"
        " wear for the duration"
    )
    duration = "1 hour"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Cleric",)


class WardingWind(Spell):
    """A strong wind (20 miles per hour) blows around you in a 10-foot
    radius and moves with you, remaining centered on you. The wind
    lasts for the spell's duration.

    The wind has the following effects:

    - It deafens you and other creatures in its area.
    - It extinguishes unprotected flames in its area that are
      torch-sized or smaller.
    - The area is difficult terrain for creatures other than you.
    - The attack rolls of ranged weapon attacks have disadvantage if
      they pass in or out of the wind.
    - It hedges out vapor, gas, and fog that can be dispersed by
      strong wind.

    """

    name = "Warding Wind"
    level = 2
    casting_time = "1 action"
    casting_range = "Self"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Evocation"
    classes = ("Bard", "Druid", "Sorcerer", "Wizard")


class WaterBreathing(Spell):
    """This spell grants up to ten willing creatures you can see within range the
    ability to breathe underwater until the spell ends. Affected creatures also
    retain their normal mode of respiration.
    """

    name = "Water Breathing"
    level = 3
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A short reed or piece of straw"
    duration = "24 hours"
    ritual = True
    magic_school = "Transmutation"
    classes = ("Druid", "Ranger", "Sorcerer", "Wizard")


class WaterWalk(Spell):
    """This spell grants the ability to move across any liquid surface – such as water,
    acid, mud, snow, quicksand, or lava – as if it were harmless solid ground
    (creatures crossing molten lava can still take damage from the heat).
    Up to ten
    willing creatures you can see within range gain this ability for the duration.


    If you target a creature submerged in a liquid, the spell carries the target to
    the surface of the liquid at a rate of 60 feet per round.
    """

    name = "Water Walk"
    level = 3
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A piece of cork"
    duration = "1 hour"
    ritual = True
    magic_school = "Transmutation"
    classes = ("Cleric", "Druid", "Ranger", "Sorcerer")


class WaterySphere(Spell):
    """You conjure up a sphere of water with a 5-foot radius on a point
    you can see within range. The sphere can hover in the air, but no
    more than 10 feet off the ground. The sphere remains for the
    spell's duration.

    Any creature in the sphere's space must make a Strength saving
    throw. On a successful save, a creature is ejected from that space
    to the nearest unoccupied space outside it.  A Huge or larger
    creature succeeds on the saving throw automatically. On a failed
    save, a creature is restrained by the sphere and is engulfed by
    the water. At the end of each of its turns, a restrained target
    can repeat the saving throw.

    The sphere can restrain a maximum of four Medium or smaller
    creatures or one Large creature. If the sphere restrains a
    creature in excess of these numbers, a random creature that was
    already restrained by the sphere falls out of it and lands prone
    in a space within 5 feet of it.

    As an action, you can move the sphere up to 30 feet in a straight
    line. If it moves over a pit, cliff, or other drop, it safely
    descends until it is hovering 10 feet over ground. Any creature
    restrained by the sphere moves with it. You can ram the sphere
    into creatures, forcing them to make the saving throw, but no more
    than once per turn.

    When the spell ends, the sphere falls to the ground and
    extinguishes all normal flames within 30 feet of it. Any creature
    restrained by the sphere is knocked prone in the space where it
    falls.

    """

    name = "Watery Sphere"
    level = 4
    casting_time = "1 action"
    casting_range = "90 feet"
    components = ("V", "S", "M")
    materials = "A droplet of water"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Druid", "Sorcerer", "Wizard")


class Web(Spell):
    """You conjure a mass of thick, sticky webbing at a point of your
    choice within range.  The webs fill a 20-foot cube from that point
    for the duration. The webs are difficult terrain and lightly
    obscure their area.

    If the webs aren't anchored between two solid masses (such as
    walls or trees) or layered across a floor, wall, or ceiling, the
    conjured web collapses on itself, and the spell ends at the start
    of your next turn. Webs layered over a flat surface have a depth
    of 5 feet.

    Each creature that starts its turn in the webs or that enters them
    during its turn must make a Dexterity saving throw. On a failed
    save, the creature is restrained as long as it remains in the webs
    or until it breaks free.

    A creature restrained by the webs can use its action to make a
    Strength check against your spell save DC. If it succeeds, it is
    no longer restrained.

    The webs are flammable. Any 5-foot cube of webs exposed to fire
    burns away in 1 round, dealing 2d4 fire damage to any creature
    that starts its turn in the fire.

    """

    name = "Web"
    level = 2
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = "A bit of spiderweb"
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Sorcerer", "Wizard")


class Weird(Spell):
    """Drawing on the deepest fears of a group of creatures, you create
    illusory creatures in their minds, visible only to them.  Each
    creature in a 30-foot-radius sphere centered on a point of your
    choice within range must make a Wisdom saving throw. On a failed
    save, a creature becomes frightened for the duration.

    The illusion calls on the creature's deepest fears, manifesting
    its worst nightmares as an implacable threat. At the end of each
    of the frightened creature's turns, it must succeed on a Wisdom
    saving throw or take 4d10 psychic damage. On a successful save,
    the spell ends for that creature.

    """

    name = "Weird"
    level = 9
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Illusion"
    classes = ("Wizard",)


class Whirlwind(Spell):
    """A whirlwind howls down to a point that you can see on the ground
    within range.  The whirlwind is a 10-foot-radius, 30-foot-high
    cylinder centered on that point.  Until the spell ends, you can
    use your action to move the whirlwind up to 30 feet in any
    direction along the ground. The whirlwind sucks up any Medium or
    smaller objects that aren't secured to anything and that aren't
    worn or carried by anyone.

    A creature must make a Dexterity saving throw the first time on a
    turn that it enters the whirlwind or that the whirlwind enters its
    space, including when the whirlwind first appears. A creature
    takes 10d6 bludgeoning damage on a failed save, or half as much
    damage on a successful one. In addition, a Large or smaller
    creature that fails the save must succeed on a Strength saving
    throw or become restrained in the whirlwind until the spell
    ends. When a creature starts its turn restrained by the whirlwind,
    the creature is pulled 5 feet higher inside it, unless the
    creature is at the top.

    A restrained creature moves with the whirlwind and falls when the
    spell ends, unless the creature has some means to stay aloft. A
    restrained creature can use an action to make a Strength or
    Dexterity check against your spell save DC. If successful, the
    creature is no longer restrained by the whirlwind and is hurled
    3d6 x 10 feet away from it in a random direction.

    """

    name = "Whirlwind"
    level = 7
    casting_time = "1 action"
    casting_range = "300 feet"
    components = ("V", "M")
    materials = "A piece of straw"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Wizard", "Sorcerer")


class WildCunning(Spell):
    """You call out to the spirits of nature to aid you. When you cast this
    spell, choose one of the following effects:
    -- If there are any tracks on the ground within range, you know where they
    are, and you make Wisdom (Survival) checks to follow these tracks with
    advantage for 1 hour or until you cast this spell again.

    -- If there is edible forage within range, you know it and where to find
    it.

    -- If there is clean drinking water within range, you know it and where to
    find it.

    -- If there is suitable shelter for you and your companions with range, you
    know it and where to find it.

    -- Send the spirits to bring back wood for a fire and to set up a campsite
    in the area using your supplies. The spirits build the fire in a circle of
    stones, put up tents, unroll bedrolls, and put out any rations and water
    for consumption.

    -- Have the spirits instantly break down a campsite, which includes putting
    out a fire, taking down tents, packing up bags, and burying any rubbish.
    """

    name = "Wild Cunning"
    level = 1
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Insantaneous"
    magic_school = "Transmutation"
    classes = ("Druid", "Ranger")


class WindWalk(Spell):
    """You and up to ten willing creatures you can see within range assume
    a gaseous form for the duration, appearing as wisps of cloud.
    While in this cloud form, a creature has a flying speed of 300
    feet and has resistance to damage from nonmagical weapons. The
    only actions a creature can take in this form are the Dash action
    or to revert to its normal form.  Reverting takes 1 minute, during
    which time a creature is incapacitated and can't move. Until the
    spell ends, a creature can revert to cloud form, which also
    requires the 1-minute transformation.

    If a creature is in cloud form and flying when the effect ends,
    the creature descends 60 feet per round for 1 minute until it
    lands, which it does safely. If it can't land after 1 minute, the
    creature falls the remaining distance.

    """

    name = "Wind Walk"
    level = 6
    casting_time = "1 minute"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "Fire and holy water"
    duration = "8 hours"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Druid",)


class WindWall(Spell):
    """A wall of strong wind rises from the ground at a point you choose
    within range.

    You can make the wall up to 50 feet long, 15 feet high, and 1 foot
    thick. You can shape the wall in any way you choose so long as it
    makes one continuous path along the ground. The wall lasts for the
    duration.

    When the wall appears, each creature within its area must make a
    Strength saving throw. A creature takes 3d8 bludgeoning damage on
    a failed save, or half as much damage on a successful one.

    The strong wind keeps fog, smoke, and other gases at bay. Small or
    smaller flying creatures or objects can't pass through the
    wall. Loose, lightweight materials brought into the wall fly
    upward. Arrows, bolts, and other ordinary projectiles launched at
    targets behind the wall are deflected upward and automatically
    miss. (Boulders hurled by giants or siege engines, and similar
    projectiles, are unaffected.) Creatures in gaseous form can't pass
    through it.

    """

    name = "Wind Wall"
    level = 3
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S", "M")
    materials = "A tiny fan and a feather of exotic origin"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Ranger")


class Wish(Spell):
    """Wish is the mightiest spell a mortal creature can cast. By simply
    speaking aloud, you can alter the very foundations of reality in
    accord with your desires.

    The basic use of this spell is to duplicate any other spell of 8th
    level or lower. You don't need to meet any requirements in that
    spell, including costly components. The spell simply takes effect.
    Alternatively, you can create one of the following effects of your
    choice:

    - You create one object of up to 25,000 gp in value that isn't a
      magic item. The object can be no more than 300 feet in any
      dimension, and it appears in an unoccupied space you can see on
      the ground.
    - You allow up to twenty creatures that you can see to regain all
      hit points, and you end all effects on them described in the
      greater restoration spell.
    - You grant up to ten creatures that you can see resistance to a
      damage type you choose.
    - You grant up to ten creatures you can see immunity to a single
      spell or other magical effect for 8 hours. For instance, you
      could make yourself and all your com panions immune to a lich's
      life drain attack.
    - You undo a single recent event by forcing a reroll of any roll
      made within the last round (including your last turn). Reality
      reshapes itself to accommodate the new result. For example, a
      wish spell could undo an opponent's successful save, a foe's
      critical hit, or a friend's failed save. You can force the
      reroll to be made with advantage or disadvantage, and you can
      choose whether to use the reroll or the original roll.

    You might be able to achieve something beyond the scope of the
    above examples. State your wish to the DM as precisely as
    possible.  The DM has great latitude in ruling what occurs in such
    an instance; the greater the wish, the greater the likelihood that
    something goes wrong. This spell might simply fail, the effect you
    desire might only be partly achieved, or you might suffer some
    unforeseen consequence as a result of how you worded the wish. For
    example, wishing that a villain were dead might propel you forward
    in time to a period when that villain is no longer alive,
    effectively removing you from the game. Similarly, wishing for a
    legendary magic item or artifact might instantly transport you to
    the presence of the item's current owner.

    The stress of casting this spell to produce any effect other than
    duplicating another spell weakens you. After enduring that stress,
    each time you cast a spell until you finish a long rest, you take
    1d10 necrotic damage per level of that spell. This damage can't be
    reduced or prevented in any way. In addition, your Strength drops
    to 3, if it isn't 3 or lower already, for 2d4 days. For each of
    those days that you spend resting and doing nothing more than
    light activity, your remaining recovery time decreases by 2
    days. Finally, there is a 33 percent chance that you are unable to
    cast wish ever again if you suffer this stress.

    """

    name = "Wish"
    level = 9
    casting_time = "1 action"
    casting_range = "Self"
    components = ("V",)
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Sorcerer", "Wizard")


class WitchBolt(Spell):
    """A beam of crackling, blue energy lances out toward a creature within range,
    forming a sustained arc of lightning between you and the target.

    Make a ranged spell attack against that creature. On a hit, the
    target takes 1d12 lightning damage, and on each of your turns for
    the duration, you can use your action to deal 1d12 lightning
    damage to the target automatically. The spell ends if you use your
    action to do anything else. The spell also ends if the target is
    ever outside the spell's range or if it has total cover from you.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 2nd level or higher, the initial damage increases by 1d12 for
    each slot level above 1st.

    """

    name = "Witch Bolt"
    level = 1
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A twig from a tree that has been struck by lightning"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Sorcerer", "Warlock", "Wizard")


class WordOfRadiance(Spell):
    """You utter a divine word, and burning radiance erupts from you. Each
    creature of your choice that you can see within range must succeed
    on a Constitution saving throw or take 1d6 radiant damage.

    The spell's damage increases by 1d6 when you reach 5th level
    (2d6), 11th level (3d6), and 17th level (4d6).

    """

    name = "Word Of Radiance"
    level = 0
    casting_time = "1 action"
    casting_range = "5 feet"
    components = ("V", "M")
    materials = "A holy symbol"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Cleric",)


class WordOfRecall(Spell):
    """You and up to five willing creatures within 5 feet of you instantly
    teleport to a previously designated sanctuary.  You and any
    creatures that teleport with you appear in the nearest unoccupied
    space to the spot you designated when you prepared your sanctuary
    (see below). If you cast this spell without first preparing a
    sanctuary, the spell has no effect.

    You must designate a sanctuary by casting this spell within a
    location, such as a temple, dedicated to or strongly linked to
    your deity. If you attempt to cast the spell in this manner in an
    area that isn’t dedicated to your deity, the spell has no effect.

    """

    name = "Word Of Recall"
    level = 6
    casting_time = "1 action"
    casting_range = "5 feet"
    components = ("V",)
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Cleric",)


class WrathOfNature(Spell):
    """You call out to the spirits of nature to rouse them against your enemies. Choose
    a point you can see within range. The spirits cause trees, rocks, and grasses
    in a 60-foot cube centered on that point to become animated until the spell
    ends.
    Grasses and Undergrowth. Any area of ground in the cube that is covered by
    grass or undergrowth is difficult terrain for your enemies.
    Trees. At the start
    of each of your turns, each of your enemies within 10 feet of any tree in the
    cube must succeed on a Dexterity saving throw or take 4d6 slashing damage from
    whipping branches.
    Roots and Vines. At the end of each of your turns, one
    creature of your choice that is on the ground in the cube must succeed on a
    Strength saving throw or become restrained until the spell ends. A restrained
    creature can use an action to make a Strength (Athletics) check against your
    spell save DC, ending the effect on itself on a success.
    Rocks. As a bonus
    action on your turn, you can cause a loose rock in the cube to launch at a
    creature you can see in the cube. Make a ranged spell attack against the target.
    On a hit, the target takes 3d8 nonmagical bludgeoning damage, and it must
    succeed on a Strength saving throw or fall prone.
    """

    name = "Wrath Of Nature"
    level = 5
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Ranger")


class WrathfulSmite(Spell):
    """The next time you hit with a melee weapon attack during this
    spell's duration, your attack deals an extra 1d6 psychic damage.
    Additionally, if the target is a creature, it must make a Wisdom
    saving throw or be frightened of you until the spell ends. As an
    action, the creature can make a Wisdom check against your spell
    save DC to steel its resolve and end this spell.
    """

    name = "Wrathful Smite"
    level = 1
    casting_time = "1 bonus action"
    casting_range = "Self"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Paladin",)


class Wristpocket(Spell):
    """You flick your wrist, causing one object in your hand to vanish.
    The object, which only you can be holding and can weigh no more than
    5 pounds, is transported to an extradimensional space, where it remains for
    the duration.
    Until the spell ends, you can use your action to summon the object to your
    free hand, and you can use your action to return the object to
    the extradimensional space. An object still in the pocket plane when
    the spell ends appears in your space, at your feet.
    """

    name = "Wristpocket"
    level = 2
    casting_time = "1 action"
    casting_range = "Self"
    components = ("S",)
    materials = ""
    duration = "Concentration, up to 1 hour"
    ritual = True
    magic_school = "Dunamancy"
    classes = ("Wizard",)



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class KillerWhale(Monster):
    """
    Echolocation.
      The whale can't use its blindsight while deafened.
    Hold Breath.
      The whale can hold its breath for 30 minutes
    Keen Hearing.
      The whale has advantage on Wisdom (Perception) checks that rely on
      hearing.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 21 (5d6
      + 4) piercing damage.
    """
    name = "Killer Whale"
    description = "Huge beast, unaligned"
    challenge_rating = 3
    armor_class = 12
    skills = "Perception +3"
    senses = "Blindsight 120 ft., Passive Perception 13"
    languages = ""
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(13)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 0
    swim_speed = 60
    fly_speed = 0
    climb_speed = 0
    hp_max = 90
    hit_dice = "12d12"
    spells = []


class Knight(Monster):
    """
    Brave.
      The knight has advantage on saving throws against being frightened.
    Multiattack.
      The knight makes two melee attacks.
    Greatsword.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) slashing damage.
    Heavy Crossbow.
      Ranged Weapon Attack: +2 to hit, range 100/400 ft., one target. Hit: 5
      (1d10) piercing damage.
    Leadership.
      For 1 minute, the knight can utter a special command or warning
      whenever a nonhostile creature that it can see within 30 ft. of it
      makes an attack roll or a saving throw. The creature can add a d4 to
      its roll provided it can hear and understand the knight. A creature
      can benefit from only one Leadership die at a time. This effect ends
      if the knight is incapacitated.
    """
    name = "Knight"
    description = "Medium humanoid, any alignment"
    challenge_rating = 3
    armor_class = 18
    skills = ""
    senses = "Passive Perception 10"
    languages = "any one language (usually Common)"
    strength = Ability(16)
    dexterity = Ability(11)
    constitution = Ability(14)
    intelligence = Ability(11)
    wisdom = Ability(11)
    charisma = Ability(15)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 52
    hit_dice = "8d8"
    spells = []


class Kobold(Monster):
    """
    Sunlight Sensitivity.
      While in sunlight, the kobold has disadvantage on attack rolls, as
      well as on Wisdom (Perception) checks that rely on sight.
    Pack Tactics.
      The kobold has advantage on an attack roll against a creature if at
      least one of the kobold's allies is within 5 ft. of the creature and
      the ally isn't incapacitated.
    Dagger.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 4 (1d4 +
      2) piercing damage.
    Sling.
      Ranged Weapon Attack: +4 to hit, range 30/120 ft., one target. Hit: 4
      (1d4 + 2) bludgeoning damage.
    """
    name = "Kobold"
    description = "Small humanoid, lawful evil"
    challenge_rating = 0.125
    armor_class = 12
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 8"
    languages = "Common, Draconic"
    strength = Ability(7)
    dexterity = Ability(15)
    constitution = Ability(9)
    intelligence = Ability(8)
    wisdom = Ability(7)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 5
    hit_dice = "2d6"
    spells = []


class Kraken(Monster):
    """
    Amphibious.
      The kraken can breathe air and water.
    Freedom of Movement.
      The kraken ignores difficult terrain, and magical effects can't reduce
      its speed or cause it to be restrained. It can spend 5 feet of
      movement to escape from nonmagical restraints or being grappled.
    Siege Monster.
      The kraken deals double damage to objects and structures.
    Multiattack.
      The kraken makes three tentacle attacks, each of which it can replace
      with one use of Fling.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 23 (3d8
      + 10) piercing damage. If the target is a Large or smaller creature
      grappled by the kraken, that creature is swallowed, and the grapple
      ends. While swallowed, the creature is blinded and restrained, it has
      total cover against attacks and other effects outside the kraken, and
      it takes 42 (12d6) acid damage at the start of each of the kraken's
      turns. If the kraken takes 50 damage or more on a single turn from a
      creature inside it, the kraken must succeed on a DC 25 Constitution
      saving throw at the end of that turn or regurgitate all swallowed
      creatures, which fall prone in a space within 10 feet of the kraken.
      If the kraken dies, a swallowed creature is no longer restrained by it
      and can escape from the corpse using 15 feet of movement, exiting
      prone.
    Tentacle.
      Melee Weapon Attack: +7 to hit, reach 30 ft., one target. Hit: 20 (3d6
      + 10) bludgeoning damage, and the target is grappled (escape DC 18).
      Until this grapple ends, the target is restrained. The kraken has ten
      tentacles, each of which can grapple one target.
    Fling.
      One Large or smaller object held or creature grappled by the kraken is
      thrown up to 60 feet in a random direction and knocked prone. If a
      thrown target strikes a solid surface, the target takes 3 (1d6)
      bludgeoning damage for every 10 feet it was thrown. If the target is
      thrown at another creature, that creature must succeed on a DC 18
      Dexterity saving throw or take the same damage and be knocked prone.
    Lightning Storm.
      The kraken magically creates three bolts of lightning, each of which
      can strike a target the kraken can see within 120 feet of it. A target
      must make a DC 23 Dexterity saving throw, taking 22 (4d10) lightning
      damage on a failed save, or half as much damage on a successful one.
    """
    name = "Kraken"
    description = "Gargantuan monstrosity, chaotic evil"
    challenge_rating = 23
    armor_class = 18
    skills = ""
    senses = "Truesight 120 ft., Passive Perception 14"
    languages = "understands Abyssal, Celestial, Infernal, and Primordial but can't speak, telepathy 120 ft."
    strength = Ability(30)
    dexterity = Ability(11)
    constitution = Ability(25)
    intelligence = Ability(22)
    wisdom = Ability(18)
    charisma = Ability(20)
    speed = 20
    swim_speed = 60
    fly_speed = 0
    climb_speed = 0
    hp_max = 472
    hit_dice = "27d20"
    spells = []



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Warhorse(Monster):
    """
    Trampling Charge.
      If the horse moves at least 20 ft. straight toward a creature and then
      hits it with a hooves attack on the same turn, that target must
      succeed on a DC 14 Strength saving throw or be knocked prone. If the
      target is prone, the horse can make another attack with its hooves
      against it as a bonus action.
    Hooves.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) bludgeoning damage.
    """
    name = "Warhorse"
    description = "Large beast, unaligned"
    challenge_rating = 0.5
    armor_class = 11
    skills = ""
    senses = "Passive Perception 11"
    languages = ""
    strength = Ability(18)
    dexterity = Ability(12)
    constitution = Ability(13)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 60
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 19
    hit_dice = "3d10"
    spells = []


class WarhorseSkeleton(Monster):
    """
    Hooves.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) bludgeoning damage.
    """
    name = "Warhorse Skeleton"
    description = "Large undead, lawful evil"
    challenge_rating = 0.5
    armor_class = 13
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 9"
    languages = ""
    strength = Ability(18)
    dexterity = Ability(12)
    constitution = Ability(15)
    intelligence = Ability(2)
    wisdom = Ability(8)
    charisma = Ability(5)
    speed = 60
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 22
    hit_dice = "3d10"
    spells = []


class WaterElemental(Monster):
    """
    Water Form.
      The elemental can enter a hostile creature's space and stop there. It
      can move through a space as narrow as 1 inch wide without squeezing.
    Freeze.
      If the elemental takes cold damage, it partially freezes; its speed is
      reduced by 20 ft. until the end of its next turn.
    Multiattack.
      The elemental makes two slam attacks.
    Slam.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) bludgeoning damage.
    Whelm.
      Each creature in the elemental's space must make a DC 15 Strength
      saving throw. On a failure, a target takes 13 (2d8 + 4) bludgeoning
      damage. If it is Large or smaller, it is also grappled (escape DC 14).
      Until this grapple ends, the target is restrained and unable to
      breathe unless it can breathe water. If the saving throw is
      successful, the target is pushed out of the elemental's space.
      
      The elemental can grapple one Large creature or up to two Medium or
      smaller creatures at one time. At the start of each of the elemental's
      turns, each target grappled by it takes 13 (2d8 + 4) bludgeoning
      damage. A creature within 5 feet of the elemental can pull a creature
      or object out of it by taking an action to make a DC 14 Strength and
      succeeding.
    """
    name = "Water Elemental"
    description = "Large elemental, neutral"
    challenge_rating = 5
    armor_class = 14
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Aquan"
    strength = Ability(18)
    dexterity = Ability(14)
    constitution = Ability(18)
    intelligence = Ability(5)
    wisdom = Ability(10)
    charisma = Ability(8)
    speed = 30
    swim_speed = 90
    fly_speed = 0
    climb_speed = 0
    hp_max = 114
    hit_dice = "12d10"
    spells = []


class Weasel(Monster):
    """
    Keen Hearing and Smell.
      The weasel has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one creature. Hit: 1
      piercing damage.
    """
    name = "Weasel"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 13
    skills = "Perception +3, Stealth +5"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(3)
    dexterity = Ability(16)
    constitution = Ability(8)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(3)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 1
    hit_dice = "1d4"
    spells = []


class WerebearBearform(Monster):
    """
    Shapechanger.
      The werebear can use its action to polymorph into a Large bear-
      humanoid hybrid or into a Large bear, or back into its true form,
      which is humanoid. Its statistics, other than its size and AC, are the
      same in each form. Any equipment it is wearing or carrying isn't
      transformed. It reverts to its true form if it dies.
    Keen Smell.
      The werebear has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack.
      In bear form, the werebear makes two claw attacks. In humanoid form,
      it makes two greataxe attacks. In hybrid form, it can attack like a
      bear or a humanoid.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 15 (2d10
      + 4) piercing damage. If the target is a humanoid, it must succeed on
      a DC 14 Constitution saving throw or be cursed with werebear
      lycanthropy.
    Claw.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) slashing damage.
    """
    name = "Werebear, Bear form"
    description = "Medium humanoid, neutral good"
    challenge_rating = 5
    armor_class = 10
    skills = "Perception +7"
    senses = "Passive Perception 17"
    languages = ""
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(17)
    intelligence = Ability(11)
    wisdom = Ability(12)
    charisma = Ability(12)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 135
    hit_dice = "18d8"
    spells = []


class WerebearHumanform(Monster):
    """
    Shapechanger.
      The werebear can use its action to polymorph into a Large bear-
      humanoid hybrid or into a Large bear, or back into its true form,
      which is humanoid. Its statistics, other than its size and AC, are the
      same in each form. Any equipment it is wearing or carrying isn't
      transformed. It reverts to its true form if it dies.
    Keen Smell.
      The werebear has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack.
      In bear form, the werebear makes two claw attacks. In humanoid form,
      it makes two greataxe attacks. In hybrid form, it can attack like a
      bear or a humanoid.
    Greataxe.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 10 (1d12
      + 4) slashing damage.
    """
    name = "Werebear, Human form"
    description = "Medium humanoid, neutral good"
    challenge_rating = 5
    armor_class = 10
    skills = "Perception +7"
    senses = "Passive Perception 17"
    languages = "Common"
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(17)
    intelligence = Ability(11)
    wisdom = Ability(12)
    charisma = Ability(12)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 135
    hit_dice = "18d8"
    spells = []


class WerebearHybridform(Monster):
    """
    Shapechanger.
      The werebear can use its action to polymorph into a Large bear-
      humanoid hybrid or into a Large bear, or back into its true form,
      which is humanoid. Its statistics, other than its size and AC, are the
      same in each form. Any equipment it is wearing or carrying isn't
      transformed. It reverts to its true form if it dies.
    Keen Smell.
      The werebear has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack.
      In bear form, the werebear makes two claw attacks. In humanoid form,
      it makes two greataxe attacks. In hybrid form, it can attack like a
      bear or a humanoid.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 15 (2d10
      + 4) piercing damage. If the target is a humanoid, it must succeed on
      a DC 14 Constitution saving throw or be cursed with werebear
      lycanthropy.
    Claw.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) slashing damage.
    Greataxe.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 10 (1d12
      + 4) slashing damage.
    """
    name = "Werebear, Hybrid form"
    description = "Medium humanoid, neutral good"
    challenge_rating = 5
    armor_class = 10
    skills = "Perception +7"
    senses = "Passive Perception 17"
    languages = "Common"
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(17)
    intelligence = Ability(11)
    wisdom = Ability(12)
    charisma = Ability(12)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 135
    hit_dice = "18d8"
    spells = []


class WereboarBoarform(Monster):
    """
    Shapechanger.
      The wereboar can use its action to polymorph into a boar-humanoid
      hybrid or into a boar, or back into its true form, which is humanoid.
      Its statistics, other than its AC, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Charge (Boar or Hybrid Form Only).
      If the wereboar moves at least 15 feet straight toward a target and
      then hits it with its tusks on the same turn, the target takes an
      extra 7 (2d6) slashing damage. If the target is a creature, it must
      succeed on a DC 13 Strength saving throw or be knocked prone.
    Relentless.
      If the wereboar takes 14 damage or less that would reduce it to 0 hit
      points, it is reduced to 1 hit point instead.
    Tusks.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) slashing damage. If the target is a humanoid, it must succeed on
      a DC 12 Constitution saving throw or be cursed with wereboar
      lycanthropy.
    """
    name = "Wereboar, Boar form"
    description = "Medium humanoid, neutral evil"
    challenge_rating = 4
    armor_class = 10
    skills = "Perception +2"
    senses = "Passive Perception 12"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(10)
    constitution = Ability(15)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(8)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 78
    hit_dice = "12d8"
    spells = []


class WereboarHumanform(Monster):
    """
    Shapechanger.
      The wereboar can use its action to polymorph into a boar-humanoid
      hybrid or into a boar, or back into its true form, which is humanoid.
      Its statistics, other than its AC, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Relentless.
      If the wereboar takes 14 damage or less that would reduce it to 0 hit
      points, it is reduced to 1 hit point instead.
    Multiattack.
      The wereboar makes two attacks, only one of which can be with its
      tusks.
    Maul.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) bludgeoning damage.
    """
    name = "Wereboar, Human form"
    description = "Medium humanoid, neutral evil"
    challenge_rating = 4
    armor_class = 10
    skills = "Perception +2"
    senses = "Passive Perception 12"
    languages = "Common (can't speak in boar form)"
    strength = Ability(17)
    dexterity = Ability(10)
    constitution = Ability(15)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 78
    hit_dice = "12d8"
    spells = []


class WereboarHybridform(Monster):
    """
    Shapechanger.
      The wereboar can use its action to polymorph into a boar-humanoid
      hybrid or into a boar, or back into its true form, which is humanoid.
      Its statistics, other than its AC, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Charge (Boar or Hybrid Form Only).
      If the wereboar moves at least 15 feet straight toward a target and
      then hits it with its tusks on the same turn, the target takes an
      extra 7 (2d6) slashing damage. If the target is a creature, it must
      succeed on a DC 13 Strength saving throw or be knocked prone.
    Relentless.
      If the wereboar takes 14 damage or less that would reduce it to 0 hit
      points, it is reduced to 1 hit point instead.
    Multiattack.
      The wereboar makes two attacks, only one of which can be with its
      tusks.
    Maul.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) bludgeoning damage.
    Tusks.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) slashing damage. If the target is a humanoid, it must succeed on
      a DC 12 Constitution saving throw or be cursed with wereboar
      lycanthropy.
    """
    name = "Wereboar, Hybrid form"
    description = "Medium humanoid, neutral evil"
    challenge_rating = 4
    armor_class = 10
    skills = "Perception +2"
    senses = "Passive Perception 12"
    languages = "Common"
    strength = Ability(17)
    dexterity = Ability(10)
    constitution = Ability(15)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 78
    hit_dice = "12d8"
    spells = []


class WereratHumanform(Monster):
    """
    Shapechanger.
      The wererat can use its action to polymorph into a rat-humanoid hybrid
      or into a giant rat, or back into its true form, which is humanoid.
      Its statistics, other than its size, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Smell.
      The wererat has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack (Humanoid or Hybrid Form Only).
      The wererat makes two attacks, only one of which can be a bite.
    Shortsword.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    Hand Crossbow.
      Ranged Weapon Attack: +4 to hit, range 30/120 ft., one target. Hit: 5
      (1d6 + 2) piercing damage.
    """
    name = "Wererat, Human form"
    description = "Medium humanoid, lawful evil"
    challenge_rating = 2
    armor_class = 12
    skills = "Perception +2, Stealth +4"
    senses = "Passive Perception 12"
    languages = "Common"
    strength = Ability(10)
    dexterity = Ability(15)
    constitution = Ability(12)
    intelligence = Ability(11)
    wisdom = Ability(10)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 33
    hit_dice = "6d8"
    spells = []


class WereratHybridform(Monster):
    """
    Shapechanger.
      The wererat can use its action to polymorph into a rat-humanoid hybrid
      or into a giant rat, or back into its true form, which is humanoid.
      Its statistics, other than its size, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Smell.
      The wererat has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack.
      The wererat makes two attacks, only one of which can be a bite.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 4 (1d4 +
      2) piercing damage. If the target is a humanoid, it must succeed on a
      DC 11 Constitution saving throw or be cursed with wererat lycanthropy.
    Shortsword.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    Hand Crossbow.
      Ranged Weapon Attack: +4 to hit, range 30/120 ft., one target. Hit: 5
      (1d6 + 2) piercing damage.
    """
    name = "Wererat, Hybrid form"
    description = "Medium humanoid, lawful evil"
    challenge_rating = 2
    armor_class = 12
    skills = "Perception +2, Stealth +4"
    senses = "Passive Perception 12"
    languages = "Common"
    strength = Ability(10)
    dexterity = Ability(15)
    constitution = Ability(12)
    intelligence = Ability(11)
    wisdom = Ability(10)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 33
    hit_dice = "6d8"
    spells = []


class WereratRatform(Monster):
    """
    Shapechanger.
      The wererat can use its action to polymorph into a rat-humanoid hybrid
      or into a giant rat, or back into its true form, which is humanoid.
      Its statistics, other than its size, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Smell.
      The wererat has advantage on Wisdom (Perception) checks that rely on
      smell.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 4 (1d4 +
      2) piercing damage. If the target is a humanoid, it must succeed on a
      DC 11 Constitution saving throw or be cursed with wererat lycanthropy.
    """
    name = "Wererat, Rat form"
    description = "Medium humanoid, lawful evil"
    challenge_rating = 2
    armor_class = 12
    skills = "Perception +2, Stealth +4"
    senses = "Darkvision 60 ft., Passive Perception 12"
    languages = ""
    strength = Ability(10)
    dexterity = Ability(15)
    constitution = Ability(12)
    intelligence = Ability(11)
    wisdom = Ability(10)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 33
    hit_dice = "6d8"
    spells = []


class WeretigerHumanform(Monster):
    """
    Shapechanger.
      The weretiger can use its action to polymorph into a tiger-humanoid
      hybrid or into a tiger, or back into its true form, which is humanoid.
      Its statistics, other than its size, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Hearing and Smell.
      The weretiger has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Multiattack.
      In humanoid form, the weretiger makes two scimitar attacks or two
      longbow attacks. In hybrid form, it can attack like a humanoid or make
      two claw attacks.
    Scimitar.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 6 (1d6 +
      3) slashing damage.
    Longbow.
      Ranged Weapon Attack: +4 to hit, range 150/600 ft., one target. Hit: 6
      (1d8 + 2) piercing damage.
    """
    name = "Weretiger, Human form"
    description = "Medium humanoid, neutral"
    challenge_rating = 4
    armor_class = 12
    skills = "Perception +5, Stealth +4"
    senses = "Darkvision 60 ft., Passive Perception 15"
    languages = "Common"
    strength = Ability(17)
    dexterity = Ability(15)
    constitution = Ability(16)
    intelligence = Ability(10)
    wisdom = Ability(13)
    charisma = Ability(11)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 120
    hit_dice = "16d8"
    spells = []


class WeretigerHybridform(Monster):
    """
    Shapechanger.
      The weretiger can use its action to polymorph into a tiger-humanoid
      hybrid or into a tiger, or back into its true form, which is humanoid.
      Its statistics, other than its size, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Hearing and Smell.
      The weretiger has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pounce.
      If the weretiger moves at least 15 feet straight toward a creature and
      then hits it with a claw attack on the same turn, that target must
      succeed on a DC 14 Strength saving throw or be knocked prone. If the
      target is prone, the weretiger can make one bite attack against it as
      a bonus action.
    Multiattack.
      In humanoid form, the weretiger makes two scimitar attacks or two
      longbow attacks. In hybrid form, it can attack like a humanoid or make
      two claw attacks.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 8 (1d10
      + 3) piercing damage. If the target is a humanoid, it must succeed on
      a DC 13 Constitution saving throw or be cursed with weretiger
      lycanthropy.
    Claw.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 7 (1d8 +
      3) slashing damage.
    Scimitar.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 6 (1d6 +
      3) slashing damage.
    Longbow.
      Ranged Weapon Attack: +4 to hit, range 150/600 ft., one target. Hit: 6
      (1d8 + 2) piercing damage.
    """
    name = "Weretiger, Hybrid form"
    description = "Medium humanoid, neutral"
    challenge_rating = 4
    armor_class = 12
    skills = "Perception +5, Stealth +4"
    senses = "Darkvision 60 ft., Passive Perception 15"
    languages = "Common"
    strength = Ability(17)
    dexterity = Ability(15)
    constitution = Ability(16)
    intelligence = Ability(10)
    wisdom = Ability(13)
    charisma = Ability(11)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 120
    hit_dice = "16d8"
    spells = []


class WeretigerTigerform(Monster):
    """
    Shapechanger.
      The weretiger can use its action to polymorph into a tiger-humanoid
      hybrid or into a tiger, or back into its true form, which is humanoid.
      Its statistics, other than its size, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Hearing and Smell.
      The weretiger has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pounce.
      If the weretiger moves at least 15 feet straight toward a creature and
      then hits it with a claw attack on the same turn, that target must
      succeed on a DC 14 Strength saving throw or be knocked prone. If the
      target is prone, the weretiger can make one bite attack against it as
      a bonus action.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 8 (1d10
      + 3) piercing damage. If the target is a humanoid, it must succeed on
      a DC 13 Constitution saving throw or be cursed with weretiger
      lycanthropy.
    Claw.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 7 (1d8 +
      3) slashing damage.
    """
    name = "Weretiger, Tiger form"
    description = "Medium humanoid, neutral"
    challenge_rating = 4
    armor_class = 12
    skills = "Perception +5, Stealth +4"
    senses = "Darkvision 60 ft., Passive Perception 15"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(15)
    constitution = Ability(16)
    intelligence = Ability(10)
    wisdom = Ability(13)
    charisma = Ability(11)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 120
    hit_dice = "16d8"
    spells = []


class WerewolfHumanform(Monster):
    """
    Shapechanger.
      The werewolf can use its action to polymorph into a wolf-humanoid
      hybrid or into a wolf, or back into its true form, which is humanoid.
      Its statistics, other than its AC, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Hearing and Smell.
      The werewolf has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Multiattack.
      The werewolf makes two attacks: two with its spear (humanoid form) or
      one with its bite and one with its claws (hybrid form).
    Spear.
      Melee or Ranged Weapon Attack: +4 to hit, reach 5 ft. or range 20/60
      ft., one creature. Hit: 5 (1d6 + 2) piercing damage, or 6 (1d8 + 2)
      piercing damage if used with two hands to make a melee attack.
    """
    name = "Werewolf, Human form"
    description = "Medium humanoid, chaotic evil"
    challenge_rating = 3
    armor_class = 11
    skills = "Perception +4"
    senses = "Passive Perception 14"
    languages = "Common"
    strength = Ability(15)
    dexterity = Ability(13)
    constitution = Ability(14)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 58
    hit_dice = "9d8"
    spells = []


class WerewolfHybridform(Monster):
    """
    Shapechanger.
      The werewolf can use its action to polymorph into a wolf-humanoid
      hybrid or into a wolf, or back into its true form, which is humanoid.
      Its statistics, other than its AC, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Hearing and Smell.
      The werewolf has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Multiattack.
      The werewolf makes two attacks: two with its spear (humanoid form) or
      one with its bite and one with its claws (hybrid form).
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6 (1d8 +
      2) piercing damage. If the target is a humanoid, it must succeed on a
      DC 12 Constitution saving throw or be cursed with werewolf
      lycanthropy.
    Claws.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 7 (2d4
      + 2) slashing damage.
    """
    name = "Werewolf, Hybrid form"
    description = "Medium humanoid, chaotic evil"
    challenge_rating = 3
    armor_class = 11
    skills = "Perception +4"
    senses = "Passive Perception 14"
    languages = "Common"
    strength = Ability(15)
    dexterity = Ability(13)
    constitution = Ability(14)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 58
    hit_dice = "9d8"
    spells = []


class WerewolfWolfform(Monster):
    """
    Shapechanger.
      The werewolf can use its action to polymorph into a wolf-humanoid
      hybrid or into a wolf, or back into its true form, which is humanoid.
      Its statistics, other than its AC, are the same in each form. Any
      equipment it is wearing or carrying isn't transformed. It reverts to
      its true form if it dies.
    Keen Hearing and Smell.
      The werewolf has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6 (1d8 +
      2) piercing damage. If the target is a humanoid, it must succeed on a
      DC 12 Constitution saving throw or be cursed with werewolf
      lycanthropy.
    """
    name = "Werewolf, Wolf form"
    description = "Medium humanoid, chaotic evil"
    challenge_rating = 3
    armor_class = 11
    skills = "Perception +4"
    senses = "Passive Perception 14"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(13)
    constitution = Ability(14)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 58
    hit_dice = "9d8"
    spells = []


class WhiteDragonWyrmling(Monster):
    """
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (1d10
      + 2) piercing damage plus 2 (1d4) cold damage.
    Cold Breath.
      The dragon exhales an icy blast of hail in a 15-foot cone. Each
      creature in that area must make a DC 12 Constitution saving throw,
      taking 22 (5d8) cold damage on a failed save, or half as much damage
      on a successful one.
    """
    name = "White Dragon Wyrmling"
    description = "Medium dragon, chaotic evil"
    challenge_rating = 2
    armor_class = 16
    skills = "Perception +4, Stealth +2"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(14)
    dexterity = Ability(10)
    constitution = Ability(14)
    intelligence = Ability(5)
    wisdom = Ability(10)
    charisma = Ability(11)
    speed = 30
    swim_speed = 30
    fly_speed = 60
    climb_speed = 0
    hp_max = 32
    hit_dice = "5d8"
    spells = []


class Wight(Monster):
    """
    Sunlight Sensitivity.
      While in sunlight, the wight has disadvantage on attack rolls, as well
      as on Wisdom (Perception) checks that rely on sight.
    Multiattack.
      The wight makes two longsword attacks or two longbow attacks. It can
      use its Life Drain in place of one longsword attack.
    Life Drain.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 5 (1d6
      + 2) necrotic damage. The target must succeed on a DC 13 Constitution
      saving throw or its hit point maximum is reduced by an amount equal to
      the damage taken. This reduction lasts until the target finishes a
      long rest. The target dies if this effect reduces its hit point
      maximum to 0.
      
      A humanoid slain by this attack rises 24 hours later as a zombie under
      the wight's control, unless the humanoid is restored to life or its
      body is destroyed. The wight can have no more than twelve zombies
      under its control at one time.
    Longsword.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6 (1d8 +
      2) slashing damage, or 7 (1d10 + 2) slashing damage if used with two
      hands.
    Longbow.
      Ranged Weapon Attack: +4 to hit, range 150/600 ft., one target. Hit: 6
      (1d8 + 2) piercing damage.
    """
    name = "Wight"
    description = "Medium undead, neutral evil"
    challenge_rating = 3
    armor_class = 14
    skills = "Perception +3, Stealth +4"
    senses = "Darkvision 60 ft., Passive Perception 13"
    languages = "the languages it knew in life"
    strength = Ability(15)
    dexterity = Ability(14)
    constitution = Ability(16)
    intelligence = Ability(10)
    wisdom = Ability(13)
    charisma = Ability(15)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 45
    hit_dice = "6d8"
    spells = []


class WilloWisp(Monster):
    """
    Consume Life.
      As a bonus action, the will-o'-wisp can target one creature it can see
      within 5 ft. of it that has 0 hit points and is still alive. The
      target must succeed on a DC 10 Constitution saving throw against this
      magic or die. If the target dies, the will-o'-wisp regains 10 (3d6)
      hit points.
    Ephemeral.
      The will-o'-wisp can't wear or carry anything.
    Incorporeal Movement.
      The will-o'-wisp can move through other creatures and objects as if
      they were difficult terrain. It takes 5 (1d10) force damage if it ends
      its turn inside an object.
    Variable Illumination.
      The will-o'-wisp sheds bright light in a 5- to 20-foot radius and dim
      light for an additional number of ft. equal to the chosen radius. The
      will-o'-wisp can alter the radius as a bonus action.
    Shock.
      Melee Spell Attack: +4 to hit, reach 5 ft., one creature. Hit: 9 (2d8)
      lightning damage.
    Invisibility.
      The will-o'-wisp and its light magically become invisible until it
      attacks or uses its Consume Life, or until its concentration ends (as
      if concentrating on a spell).
    """
    name = "Will-o'-Wisp"
    description = "Tiny undead, chaotic evil"
    challenge_rating = 2
    armor_class = 19
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 12"
    languages = "the languages it knew in life"
    strength = Ability(1)
    dexterity = Ability(28)
    constitution = Ability(10)
    intelligence = Ability(13)
    wisdom = Ability(14)
    charisma = Ability(11)
    speed = 0
    swim_speed = 0
    fly_speed = 50
    climb_speed = 0
    hp_max = 22
    hit_dice = "9d4"
    spells = []


class WinterWolf(Monster):
    """
    Keen Hearing and Smell.
      The wolf has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pack Tactics.
      The wolf has advantage on an attack roll against a creature if at
      least one of the wolf's allies is within 5 ft. of the creature and the
      ally isn't incapacitated.
    Snow Camouflage.
      The wolf has advantage on Dexterity (Stealth) checks made to hide in
      snowy terrain.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) piercing damage. If the target is a creature, it must succeed on
      a DC 14 Strength saving throw or be knocked prone.
    Cold Breath.
      The wolf exhales a blast of freezing wind in a 15-foot cone. Each
      creature in that area must make a DC 12 Dexterity saving throw, taking
      18 (4d8) cold damage on a failed save, or half as much damage on a
      successful one.
    """
    name = "Winter Wolf"
    description = "Large monstrosity, neutral evil"
    challenge_rating = 3
    armor_class = 13
    skills = "Perception +5, Stealth +3"
    senses = "Passive Perception 15"
    languages = "Common, Giant, Winter Wolf"
    strength = Ability(18)
    dexterity = Ability(13)
    constitution = Ability(14)
    intelligence = Ability(7)
    wisdom = Ability(12)
    charisma = Ability(8)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 75
    hit_dice = "10d10"
    spells = []


class Wolf(Monster):
    """
    Keen Hearing and Smell.
      The wolf has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pack Tactics.
      The wolf has advantage on an attack roll against a creature if at
      least one of the wolf's allies is within 5 ft. of the creature and the
      ally isn't incapacitated.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (2d4 +
      2) piercing damage. If the target is a creature, it must succeed on a
      DC 11 Strength saving throw or be knocked prone.
    """
    name = "Wolf"
    description = "Medium beast, unaligned"
    challenge_rating = 0.25
    armor_class = 13
    skills = "Perception +3, Stealth +4"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(12)
    dexterity = Ability(15)
    constitution = Ability(12)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(6)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class Worg(Monster):
    """
    Keen Hearing and Smell.
      The worg has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) piercing damage. If the target is a creature, it must succeed on
      a DC 13 Strength saving throw or be knocked prone.
    """
    name = "Worg"
    description = "Large monstrosity, neutral evil"
    challenge_rating = 0.5
    armor_class = 13
    skills = "Perception +4"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = "Goblin, Worg"
    strength = Ability(16)
    dexterity = Ability(13)
    constitution = Ability(13)
    intelligence = Ability(7)
    wisdom = Ability(11)
    charisma = Ability(8)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 26
    hit_dice = "4d10"
    spells = []


class Wraith(Monster):
    """
    Incorporeal Movement.
      The wraith can move through other creatures and objects as if they
      were difficult terrain. It takes 5 (1d10) force damage if it ends its
      turn inside an object.
    Sunlight Sensitivity.
      While in sunlight, the wraith has disadvantage on attack rolls, as
      well as on Wisdom (Perception) checks that rely on sight.
    Life Drain.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one creature. Hit: 21
      (4d8 + 3) necrotic damage. The target must succeed on a DC 14
      Constitution saving throw or its hit point maximum is reduced by an
      amount equal to the damage taken. This reduction lasts until the
      target finishes a long rest. The target dies if this effect reduces
      its hit point maximum to 0.
    Create Specter.
      The wraith targets a humanoid within 10 feet of it that has been dead
      for no longer than 1 minute and died violently. The target's spirit
      rises as a specter in the space of its corpse or in the nearest
      unoccupied space. The specter is under the wraith's control. The
      wraith can have no more than seven specters under its control at one
      time.
    """
    name = "Wraith"
    description = "Medium undead, neutral evil"
    challenge_rating = 5
    armor_class = 13
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 12"
    languages = "the languages it knew in life"
    strength = Ability(6)
    dexterity = Ability(16)
    constitution = Ability(16)
    intelligence = Ability(12)
    wisdom = Ability(14)
    charisma = Ability(15)
    speed = 0
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 67
    hit_dice = "9d8"
    spells = []


class Wyvern(Monster):
    """
    Multiattack.
      The wyvern makes two attacks: one with its bite and one with its
      stinger. While flying, it can use its claws in place of one other
      attack.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one creature. Hit: 11
      (2d6 + 4) piercing damage.
    Claws.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) slashing damage.
    Stinger.
      Melee Weapon Attack: +7 to hit, reach 10 ft., one creature. Hit: 11
      (2d6 + 4) piercing damage. The target must make a DC 15 Constitution
      saving throw, taking 24 (7d6) poison damage on a failed save, or half
      as much damage on a successful one.
    """
    name = "Wyvern"
    description = "Large dragon, unaligned"
    challenge_rating = 6
    armor_class = 13
    skills = "Perception +4"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = ""
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(16)
    intelligence = Ability(5)
    wisdom = Ability(12)
    charisma = Ability(6)
    speed = 20
    swim_speed = 0
    fly_speed = 80
    climb_speed = 0
    hp_max = 110
    hit_dice = "13d10"
    spells = []
